@@ -1,4 +1,4 @@
-/* Copyright 2004,2007 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007-2009 ENSEIRB, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -62,10 +62,22 @@
 /**                # Version 4.0  : from : 01 jan 2002     **/
 /**                                 to     07 dec 2004     **/
 /**                # Version 5.1  : from : 11 dec 2007     **/
-/**                                 to     03 apr 2008     **/
+/**                                 to     04 apr 2009     **/
 /**                                                        **/
 /************************************************************/
 
+#define ARCH_H
+
+/*
+** The defines.
+*/
+
+/*+ Architecture flags. +*/
+
+#define ARCHNONE                    0x0000        /*+ No options set                      +*/
+#define ARCHPART                    0x0001        /*+ Architecture without external gains +*/
+#define ARCHVAR                     0x0002        /*+ Variable-sized architecture         +*/
+
 /*
 **  The type and structure definitions.
 */
@@ -74,39 +86,45 @@ typedef INT Anum;                                 /*+ Generic integer +*/
 
 #define ANUMMAX                     INTVALMAX
 
-/** The domain number type. **/
+#define ANUM_MPI                    COMM_INT      /*+ MPI type for Gnum is MPI type for INT +*/
+
+/*+ The domain number type. +*/
 
 typedef Anum ArchDomNum;                          /*+ Domain number +*/
 
 #define ARCHDOMNOTTERM              ((ArchDomNum) ~0) /*+ Not-terminal number +*/
 
-/** The architecture class type. **/
+/*+ The architecture class type. +*/
 
 typedef struct ArchClass_ {
   char *                    archname;             /*+ Architecture name                 +*/
-  int                    (* archLoad)  ();        /*+ Architecture loading function     +*/
-  int                    (* archSave)  ();        /*+ Architecture saving function      +*/
-  int                    (* archFree)  ();        /*+ Architecture freeing function     +*/
-  ArchDomNum             (* domNum)    ();        /*+ Domain labeling function          +*/
-  int                    (* domTerm)   ();        /*+ Terminal domain building function +*/
-  Anum                   (* domSize)   ();        /*+ Domain size function              +*/
-  Anum                   (* domWght)   ();        /*+ Domain weight function            +*/
-  Anum                   (* domDist)   ();        /*+ Distance computation function     +*/
-  int                    (* domFrst)   ();        /*+ Compute biggest domain            +*/
-  int                    (* domLoad)   ();        /*+ Domain loading routine            +*/
-  int                    (* domSave)   ();        /*+ Domain saving routine             +*/
-  int                    (* domBipart) ();        /*+ Domain bipartitioning routine     +*/
+  int                       flagval;              /*+ Architecture flags                +*/
+  int                    (* archLoad)   ();       /*+ Architecture loading function     +*/
+  int                    (* archSave)   ();       /*+ Architecture saving function      +*/
+  int                    (* archFree)   ();       /*+ Architecture freeing function     +*/
+  ArchDomNum             (* domNum)     ();       /*+ Domain labeling function          +*/
+  int                    (* domTerm)    ();       /*+ Terminal domain building function +*/
+  Anum                   (* domSize)    ();       /*+ Domain size function              +*/
+  Anum                   (* domWght)    ();       /*+ Domain weight function            +*/
+  Anum                   (* domDist)    ();       /*+ Distance computation function     +*/
+  int                    (* domFrst)    ();       /*+ Compute biggest domain            +*/
+  int                    (* domLoad)    ();       /*+ Domain loading routine            +*/
+  int                    (* domSave)    ();       /*+ Domain saving routine             +*/
+  int                    (* domBipart)  ();       /*+ Domain bipartitioning routine     +*/
+#ifdef SCOTCH_PTSCOTCH
+  int                    (* domMpiType) ();       /*+ Domain MPI type building routine  +*/
+#endif /* SCOTCH_PTSCOTCH */
   int                       domsizeof;            /*+ Size in bytes of domain data      +*/
 } ArchClass;
 
-/** The architecture type. **/
+/*+ The architecture type. +*/
 
-typedef double ArchDummy[3];                      /*+ Size of the dummy space that can hold any architecture +*/
+typedef Anum ArchDummy[8];                        /*+ Size of the dummy space that can hold any architecture +*/
 
 typedef struct Arch_ {
-  const ArchClass *         class;                /*+ Pointer to architecture class        +*/
-  union {                                         /*+ Architecture data                    +*/
-    ArchDummy               dummy;                /*+ Dummy data for external size setting +*/
+  const ArchClass *         class;                /*+ Pointer to architecture class              +*/
+  union {                                         /*+ Architecture data                          +*/
+    ArchDummy               dummy;                /*+ Dummy data for external size setting       +*/
 #if 0
     ArchDeco                deco;                 /*+ Decomposition-described architecture       +*/
     ArchMesh2               mesh2;                /*+ 2D-mesh architecture                       +*/
@@ -123,13 +141,13 @@ typedef struct Arch_ {
   } data;
 } Arch;
 
-/** The domain structure type. **/
+/*+ The domain structure type. +*/
 
 typedef Anum ArchDomDummy[6];                     /*+ Size of the dummy space that can hold any domain +*/
 
 typedef struct ArchDom_ {
-  union {                                         /*+ The domain data                       +*/
-    ArchDomDummy            dummy;                /*+ Dummy data for external space setting +*/
+  union {                                         /*+ The domain data                           +*/
+    ArchDomDummy            dummy;                /*+ Dummy data for external space setting     +*/
 #if 0
     ArchDomDeco             deco;                 /*+ Decomposition-descripted domain type      +*/
     ArchDomMesh2            mesh2;                /*+ 2D-mesh domain type                       +*/
@@ -171,6 +189,9 @@ int                         archDomFrst         (const Arch * const, ArchDom * c
 int                         archDomLoad         (const Arch * const, ArchDom * const, FILE * const);
 int                         archDomSave         (const Arch * const, const ArchDom * const, FILE * const);
 int                         archDomBipart       (const Arch * const, const ArchDom * const, ArchDom * const, ArchDom * const);
+#ifdef SCOTCH_PTSCOTCH
+int                         archDomMpiType      (const Arch * const, MPI_Datatype * const);
+#endif /* SCOTCH_PTSCOTCH */
 
 #undef static
 
@@ -180,6 +201,8 @@ int                         archDomBipart       (const Arch * const, const ArchD
 
 #define archDomSizeof(a)            ((a)->class->domsizeof)
 #define archName(a)                 (((a)->class == NULL) ? "" : (a)->class->archname)
+#define archPart(a)                 ((((a)->class->flagval) & ARCHPART) != 0)
+#define archVar(a)                  ((((a)->class->flagval) & ARCHVAR) != 0)
 
 #if ((! defined SCOTCH_DEBUG_ARCH2) || (defined ARCH))
 #define archDomNum2(arch,dom)       (((ArchDomNum (*) (const void * const, const void * const)) (arch)->class->domNum) ((const void * const) &(arch)->data, (const void * const) &(dom)->data))
@@ -199,3 +222,38 @@ int                         archDomBipart       (const Arch * const, const ArchD
 #define archDomFrst                 archDomFrst2
 #define archDomBipart               archDomBipart2
 #endif /* SCOTCH_DEBUG_ARCH2 */
+
+#ifdef SCOTCH_PTSCOTCH
+#define ARCHCLASSBLOCK(s,n,f)       { s, f,		   \
+                                      arch##n##ArchLoad,   \
+                                      arch##n##ArchSave,   \
+                                      arch##n##ArchFree,   \
+                                      arch##n##DomNum,     \
+                                      arch##n##DomTerm,    \
+                                      arch##n##DomSize,    \
+                                      arch##n##DomWght,    \
+                                      arch##n##DomDist,    \
+                                      arch##n##DomFrst,    \
+                                      arch##n##DomLoad,    \
+                                      arch##n##DomSave,    \
+                                      arch##n##DomBipart,  \
+                                      arch##n##DomMpiType, \
+                                      sizeof (Arch##n##Dom) }
+#else /* SCOTCH_PTSCOTCH */
+#define ARCHCLASSBLOCK(s,n,f)       { s, f,		  \
+                                      arch##n##ArchLoad,  \
+                                      arch##n##ArchSave,  \
+                                      arch##n##ArchFree,  \
+                                      arch##n##DomNum,    \
+                                      arch##n##DomTerm,   \
+                                      arch##n##DomSize,   \
+                                      arch##n##DomWght,   \
+                                      arch##n##DomDist,   \
+                                      arch##n##DomFrst,   \
+                                      arch##n##DomLoad,   \
+                                      arch##n##DomSave,   \
+                                      arch##n##DomBipart, \
+                                      sizeof (Arch##n##Dom) }
+#endif /* SCOTCH_PTSCOTCH */
+
+#define ARCHCLASSBLOCKNULL          { NULL, ARCHNONE }
