@@ -58,6 +58,8 @@
 /**                                 to     01 oct 1998     **/
 /**                # Version 4.0  : from : 09 jan 2004     **/
 /**                                 to     10 mar 2005     **/
+/**                # Version 5.1  : from : 19 jan 2008     **/
+/**                                 to     21 jan 2008     **/
 /**                                                        **/
 /************************************************************/
 
@@ -299,3 +301,22 @@ ArchCmpltDom * restrict const dom1ptr)
 
   return (0);
 }
+
+/* This function creates the MPI_Datatype for
+** complete graph domains.
+** It returns:
+** - 0  : if type could be created.
+** - 1  : on error.
+*/
+
+#ifdef SCOTCH_PTSCOTCH
+int
+archCmpltDomMpiType (
+const ArchCmplt * const       archptr,
+MPI_Datatype * const          typeptr)
+{
+  MPI_Type_contiguous (2, ANUM_MPI, typeptr);
+
+  return (0);
+}
+#endif /* SCOTCH_PTSCOTCH */
