@@ -56,8 +56,8 @@
 /**                                 to     14 may 1998     **/
 /**                # Version 4.0  : from : 11 nov 2003     **/
 /**                                 to     10 mar 2005     **/
-/**                # Version 5.0  : from : 27 feb 2007     **/
-/**                                 to     27 feb 2007     **/
+/**                # Version 5.1  : from : 21 jan 2008     **/
+/**                                 to     27 feb 2008     **/
 /**                                                        **/
 /************************************************************/
 
@@ -305,3 +305,22 @@ ArchHcubDom * restrict const  dom1ptr)
 
   return (0);
 }
+
+/* This function creates the MPI_Datatype for
+** hypercube domains.
+** It returns:
+** - 0  : if type could be created.
+** - 1  : on error.
+*/
+
+#ifdef SCOTCH_PTSCOTCH
+int
+archHcubDomMpiType (
+const ArchHcub * const        archptr,
+MPI_Datatype * const          typeptr)
+{
+  MPI_Type_contiguous (2, ANUM_MPI, typeptr);
+
+  return (0);
+}
+#endif /* SCOTCH_PTSCOTCH */
