@@ -60,7 +60,7 @@
 /**                                 to     07 jun 2001     **/
 /**                # Version 4.0  : from : 10 dec 2003     **/
 /**                                 to     10 mar 2005     **/
-/**                # Version 5.0  : from : 28 feb 2008     **/
+/**                # Version 5.1  : from : 21 jan 2008     **/
 /**                                 to     28 feb 2008     **/
 /**                                                        **/
 /************************************************************/
@@ -323,3 +323,22 @@ ArchTleafDom * restrict const dom1ptr)
 
   return (0);
 }
+
+/* This function creates the MPI_Datatype for
+** tree-leaf domains.
+** It returns:
+** - 0  : if type could be created.
+** - 1  : on error.
+*/
+
+#ifdef SCOTCH_PTSCOTCH
+int
+archTleafDomMpiType (
+const ArchTleaf * const       archptr,
+MPI_Datatype * const          typeptr)
+{
+  MPI_Type_contiguous (2, ANUM_MPI, typeptr);
+
+  return (0);
+}
+#endif /* SCOTCH_PTSCOTCH */
