@@ -58,8 +58,8 @@
 /**                                 to     01 oct 1998     **/
 /**                # Version 4.0  : from : 05 nov 2003     **/
 /**                                 to     10 mar 2005     **/
-/**                # Version 5.0  : from : 27 feb 2008     **/
-/**                                 to     27 feb 2008     **/
+/**                # Version 5.1  : from : 21 jan 2008     **/
+/**                                 to     28 feb 2008     **/
 /**                                                        **/
 /************************************************************/
 
@@ -328,6 +328,25 @@ ArchTorus2Dom * restrict const  dom1ptr)
   return (0);
 }
 
+/* This function creates the MPI_Datatype for
+** 2D torus domains.
+** It returns:
+** - 0  : if type could be created.
+** - 1  : on error.
+*/
+
+#ifdef SCOTCH_PTSCOTCH
+int
+archTorus2DomMpiType (
+const ArchTorus2 * const      archptr,
+MPI_Datatype * const          typeptr)
+{
+  MPI_Type_contiguous (4, ANUM_MPI, typeptr);
+
+  return (0);
+}
+#endif /* SCOTCH_PTSCOTCH */
+
 /***********************************************/
 /*                                             */
 /* These are the 3-dimensional torus routines. */
@@ -625,3 +644,22 @@ ArchTorus3Dom * restrict const  dom1ptr)
 
   return (0);
 }
+
+/* This function creates the MPI_Datatype for
+** 3D torus domains.
+** It returns:
+** - 0  : if type could be created.
+** - 1  : on error.
+*/
+
+#ifdef SCOTCH_PTSCOTCH
+int
+archTorus3DomMpiType (
+const ArchTorus3 * const      archptr,
+MPI_Datatype * const          typeptr)
+{
+  MPI_Type_contiguous (6, ANUM_MPI, typeptr);
+
+  return (0);
+}
+#endif /* SCOTCH_PTSCOTCH */
