@@ -1,4 +1,4 @@
-/* Copyright 2004,2007 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008 ENSEIRB, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -60,6 +60,8 @@
 /**                                 to     08 nov 2001     **/
 /**                # Version 4.0  : from : 05 nov 2003     **/
 /**                                 to     05 nov 2003     **/
+/**                # Version 5.1  : from : 21 jan 2008     **/
+/**                                 to     21 jan 2008     **/
 /**                                                        **/
 /************************************************************/
 
@@ -222,3 +224,22 @@ ArchVcmpltDom * restrict const  dom1ptr)
 
   return ((dom1ptr->termnum < domptr->termnum) ? 2 : 0); /* Return error on overflow */
 }
+
+/* This function creates the MPI_Datatype for
+** variable-sized complete graph domains.
+** It returns:
+** - 0  : if type could be created.
+** - 1  : on error.
+*/
+
+#ifdef SCOTCH_PTSCOTCH
+int
+archVcmpltDomMpiType (
+const ArchVcmplt * const      archptr,
+MPI_Datatype * const          typeptr)
+{
+  *typeptr = ANUM_MPI;
+
+  return (0);
+}
+#endif /* SCOTCH_PTSCOTCH */
