@@ -42,7 +42,7 @@
 /**                                 to     08 nov 2001     **/
 /**                # Version 4.0  : from : 04 nov 2003     **/
 /**                                 to     04 nov 2003     **/
-/**                # Version 5.0  : from : 27 feb 2008     **/
+/**                # Version 5.1  : from : 21 jan 2008     **/
 /**                                 to     27 feb 2008     **/
 /**                                                        **/
 /************************************************************/
@@ -239,3 +239,22 @@ ArchVhcubDom * restrict const dom1ptr)
 
   return ((dom1ptr->termnum < domptr->termnum) ? 2 : 0); /* Return error on overflow */
 }
+
+/* This function creates the MPI_Datatype for
+** variable-sized hypercube domains.
+** It returns:
+** - 0  : if type could be created.
+** - 1  : on error.
+*/
+
+#ifdef SCOTCH_PTSCOTCH
+int
+archVhcubDomMpiType (
+const ArchVhcub * const       archptr,
+MPI_Datatype * const          typeptr)
+{
+  MPI_Type_contiguous (2, ANUM_MPI, typeptr);
+
+  return (0);
+}
+#endif /* SCOTCH_PTSCOTCH */
