@@ -1,4 +1,4 @@
-/* Copyright 2004,2007 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008 ENSEIRB, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -31,7 +31,7 @@
 */
 /************************************************************/
 /**                                                        **/
-/**   NAME       : bipart_fm.c                             **/
+/**   NAME       : bgraph_bipart_fm.c                      **/
 /**                                                        **/
 /**   AUTHOR     : Francois PELLEGRINI                     **/
 /**                                                        **/
@@ -61,6 +61,8 @@
 /**                                 to     05 may 2006     **/
 /**                # Version 5.0  : from : 24 mar 2008     **/
 /**                                 to   : 22 may 2008     **/
+/**                # Version 5.1  : from : 30 oct 2008     **/
+/**                                 to   : 12 nov 2008     **/
 /**                                                        **/
 /************************************************************/
 
@@ -99,11 +101,11 @@ GainTabl * restrict const   tablptr,              /*+ Gain table        +*/
 const Gnum                  deltcur,              /*+ Current imbalance +*/
 const Gnum                  deltmax)              /*+ Maximum imbalance +*/
 {
-  BgraphBipartFmVertex * restrict vexxptr;
-  BgraphBipartFmVertex * restrict vertbest;
-  Gnum                            gainbest;
-  const GainEntr * restrict       tablbest;
-  Gnum                            deltbest;
+  BgraphBipartFmVertex *  vexxptr;
+  BgraphBipartFmVertex *  vertbest;
+  Gnum                    gainbest;
+  const GainEntr *        tablbest;
+  Gnum                    deltbest;
 
   tablbest = tablptr->tend;                       /* Assume no candidate vertex found yet */
   gainbest = GAINMAX;
@@ -148,35 +150,40 @@ bgraphBipartFm (
 Bgraph * restrict const           grafptr,        /*+ Active graph      +*/
 const BgraphBipartFmParam * const paraptr)        /*+ Method parameters +*/
 {
-  GainTabl * restrict             tablptr;        /* Pointer to gain table                 */
-  INT                             passnbr;        /* Maximum number of passes to go        */
-  BgraphBipartFmSave * restrict   savetab;        /* Pointer to move array                 */
-  Gnum                            movenbr;        /* Number of uneffective moves done      */
-  Gnum                            savenbr;        /* Number of recorded backtrack moves    */
-  Gnum                            mswpnum;        /* Current number of recording sweep     */
-  int                             moveflag;       /* Flag set if useful moves made         */
-  int                             swapval;        /* Flag set if global swap performed     */
-  int                             swapvalbst;     /* Recorded swap value for best position */
-  Gnum                            hashsiz;        /* Size of hash table                    */
-  Gnum                            hashmsk;        /* Mask for access to hash table         */
-  Gnum                            hashnum;        /* Hash value                            */
-  BgraphBipartFmVertex *          lockptr;        /* Linked list of locked vertices        */
-  BgraphBipartFmVertex * restrict hashtab;        /* Extended vertex array                 */
+  GainTabl * restrict             tablptr;        /* Pointer to gain table                    */
+  INT                             passnbr;        /* Maximum number of passes to go           */
+  BgraphBipartFmSave * restrict   savetab;        /* Pointer to move array                    */
+  Gnum                            movenbr;        /* Number of uneffective moves done         */
+  Gnum                            savenbr;        /* Number of recorded backtrack moves       */
+  Gnum                            mswpnum;        /* Current number of recording sweep        */
+  int                             moveflag;       /* Flag set if useful moves made            */
+  int                             swapval;        /* Flag set if global swap performed        */
+  int                             swapvalbst;     /* Recorded swap value for best position    */
+  Gnum                            hashsiz;        /* Size of hash table                       */
+  Gnum                            hashmsk;        /* Mask for access to hash table            */
+  Gnum                            hashnum;        /* Hash value                               */
+  BgraphBipartFmVertex *          lockptr;        /* Linked list of locked vertices           */
+  BgraphBipartFmVertex * restrict hashtab;        /* Extended vertex array                    */
   Gnum                            hashmax;
   Gnum                            hashnbr;
+  Gnum                            compload0dltmat; /* Theoretical latgest unbalance allowed   */
+  Gnum                            compload0dltmax; /* Largest unbalance allowed               */
+  Gnum                            compload0dltbst; /* Best unbalance value found to date      */
+  Gnum                            compload0dlt;   /* Current imbalance                        */
+  Gnum                            compsize0dlt;   /* Update of size of part 0                 */
+  Gnum                            commgainextn;   /* Current external communication gain      */
+  Gnum                            commgainextnbst; /* External gain of best recorded position */
+  Gnum                            commload;       /* Communication load of current position   */
+  Gnum                            commloadbst;    /* Best communication load to date          */
+  Gnum                            domdist;        /* Distance between the two subdomains      */
+  Gnum                            fronnbr;
+  Gnum                            fronnum;
 
-  Gnum               fronnbr;
-  Gnum               compload0dltmat;            /* Theoretical latgest unbalance allowed   */
-  Gnum               compload0dltmax;            /* Largest unbalance allowed               */
-  Gnum               compload0dltbst;            /* Best unbalance value found to date      */
-  Gnum               compload0dlt;               /* Current imbalance                       */
-  Gnum               compsize0dlt;               /* Update of size of part 0                */
-  Gnum               commgainextn;               /* Current external communication gain     */
-  Gnum               commgainextnbst;            /* External gain of best recorded position */
-  Gnum               commload;                   /* Communication load of current position  */
-  Gnum               commloadbst;                /* Best communication load to date         */
-  Gnum               domdist;                    /* Distance between the two subdomains     */
-  Gnum               fronnum;
+  const Gnum * restrict const verttax = grafptr->s.verttax; /* Fast accesses */
+  const Gnum * restrict const vendtax = grafptr->s.vendtax;
+  const Gnum * restrict const velotax = grafptr->s.velotax;
+  const Gnum * restrict const edgetax = grafptr->s.edgetax;
+  const Gnum * restrict const edlotax = grafptr->s.edlotax;
 
   compload0dltmat = (paraptr->deltval > 0.0L) ? ((Gnum) (paraptr->deltval * (double) grafptr->s.velosum) + 1) : 0;
   compload0dltmax = MAX (compload0dltmat, abs (grafptr->compload0dlt)); /* Set current maximum distance */
@@ -241,16 +248,16 @@ const BgraphBipartFmParam * const paraptr)        /*+ Method parameters +*/
     vertnum = grafptr->frontab[fronnum];
     partval = grafptr->parttax[vertnum];
 
-    for (edgenum = grafptr->s.verttax[vertnum], commcut = commgain = 0, edloval = 1;
-         edgenum < grafptr->s.vendtax[vertnum]; edgenum ++) {
+    for (edgenum = verttax[vertnum], commcut = commgain = 0, edloval = 1;
+         edgenum < vendtax[vertnum]; edgenum ++) {
       Gnum                vertend;
       int                 partend;
       int                 partdlt;
 
-      vertend = grafptr->s.edgetax[edgenum];
+      vertend = edgetax[edgenum];
       partend = grafptr->parttax[vertend];
-      if (grafptr->s.edlotax != NULL)
-        edloval = grafptr->s.edlotax[edgenum];
+      if (edlotax != NULL)
+        edloval = edlotax[edgenum];
 
       partdlt   = partval ^ partend;
       commcut  += partdlt;
@@ -258,7 +265,7 @@ const BgraphBipartFmParam * const paraptr)        /*+ Method parameters +*/
     }
     commgain *= domdist;                          /* Adjust internal gains with respect to external gains */
     partdlt   = 2 * partval - 1;
-    veloval   = (grafptr->s.velotax != NULL) ? grafptr->s.velotax[vertnum] : 1;
+    veloval   = (velotax != NULL) ? velotax[vertnum] : 1;
 
     for (hashnum = (vertnum * BGRAPHBIPARTFMHASHPRIME) & hashmsk; hashtab[hashnum].vertnum != ~0; hashnum = (hashnum + 1) & hashmsk) ;
 
@@ -376,17 +383,17 @@ const BgraphBipartFmParam * const paraptr)        /*+ Method parameters +*/
       vexxptr->partval  = partval ^ 1;            /* Swap vertex first in case neighbors are added */
       vexxptr->compgain = - vexxptr->compgain;
       vexxptr->commgain = - vexxptr->commgain;
-      vexxptr->commcut  = grafptr->s.vendtax[vertnum] - grafptr->s.verttax[vertnum] - vexxptr->commcut;
+      vexxptr->commcut  = vendtax[vertnum] - verttax[vertnum] - vexxptr->commcut;
 
       edloval = 1;
-      for (edgenum = grafptr->s.verttax[vertnum]; /* (Re-)link neighbors */
-           edgenum < grafptr->s.vendtax[vertnum]; edgenum ++) {
+      for (edgenum = verttax[vertnum];            /* (Re-)link neighbors */
+           edgenum < vendtax[vertnum]; edgenum ++) {
         Gnum                vertend;              /* Number of current end neighbor vertex */
         Gnum                hashnum;
 
-        vertend = grafptr->s.edgetax[edgenum];
-        if (grafptr->s.edlotax != NULL)
-          edloval = grafptr->s.edlotax[edgenum];
+        vertend = edgetax[edgenum];
+        if (edlotax != NULL)
+          edloval = edlotax[edgenum];
 
         for (hashnum = (vertend * BGRAPHBIPARTFMHASHPRIME) & hashmsk; ; hashnum = (hashnum + 1) & hashmsk) {
           if (hashtab[hashnum].vertnum == vertend) { /* If hash slot found */
@@ -433,22 +440,22 @@ const BgraphBipartFmParam * const paraptr)        /*+ Method parameters +*/
               for (hashnum = (vertend * BGRAPHBIPARTFMHASHPRIME) & hashmsk; hashtab[hashnum].vertnum != ~0; hashnum = (hashnum + 1) & hashmsk) ; /* Search for new first free slot */
             }
 
-            if (grafptr->s.edlotax != NULL) {     /* If graph edges are weighted */
+            if (edlotax != NULL) {                /* If graph edges are weighted */
               Gnum               edgeend;
 
-              for (edgeend = grafptr->s.verttax[vertend], commgainold = 0; /* Compute neighbor edge load sum */
-                   edgeend < grafptr->s.vendtax[vertend]; edgeend ++)
-                commgainold += grafptr->s.edlotax[edgeend];
+              for (edgeend = verttax[vertend], commgainold = 0; /* Compute neighbor edge load sum */
+                   edgeend < vendtax[vertend]; edgeend ++)
+                commgainold += edlotax[edgeend];
               commgain = commgainold - 2 * edloval;
             }
             else {                                /* Graph edges are not weighted */
-              commgainold = grafptr->s.vendtax[vertend] - grafptr->s.verttax[vertend];
+              commgainold = vendtax[vertend] - verttax[vertend];
               commgain    = commgainold - 2;
             }
 
             veloval = 1;
-            if (grafptr->s.velotax != NULL)
-              veloval = grafptr->s.velotax[vertend];
+            if (velotax != NULL)
+              veloval = velotax[vertend];
             veexval = 0;
             if (grafptr->veextax != NULL)
               veexval = grafptr->veextax[vertend];
@@ -697,14 +704,16 @@ const Gnum                        savenbr,        /*+ Number of moves recorded
 GainTabl * const                  tablptr,        /*+ Gain table                 +*/
 BgraphBipartFmVertex ** const     lockptr)        /*+ Pointer to locked list     +*/
 {
-  BgraphBipartFmVertex * restrict hashtab;        /* Extended vertex array                */
-  BgraphBipartFmSave * restrict   savetab;        /* Move backtracking array              */
-  BgraphBipartFmSave * restrict   saveold;        /* Pointer to translated old save array */
+  BgraphBipartFmVertex * restrict hashtab;        /* Extended vertex array                        */
+  BgraphBipartFmSave *            savetab;        /* Move backtracking array                      */
+  BgraphBipartFmSave *            saveold;        /* Pointer to translated old save array         */
   Gnum                            savenum;
-  Gnum                            hashold;        /* Size of old hash table (half of new) */
+  Gnum                            hashold;        /* Size of old hash table (half of new)         */
   Gnum                            hashsiz;
   Gnum                            hashmax;
   Gnum                            hashmsk;
+  Gnum                            hashsta;        /* Start index of range of hash indices to move */
+  Gnum                            hashend;        /* End index of range of hash indices to move   */
   Gnum                            hashnum;
 
   hashmax = *hashmaxptr << 1;                     /* Compute new sizes */
@@ -745,54 +754,45 @@ BgraphBipartFmVertex ** const     lockptr)        /*+ Pointer to locked list
   gainTablFree (tablptr);                         /* Reset gain table  */
   *lockptr = NULL;                                /* Rebuild lock list */
 
-  if (hashtab[0].vertnum != ~0) {                 /* If vertex overflowing may have occured in old hash table */
-    Gnum                        hashtmp;
-    Gnum                        hashnew;
+  for (hashsta = hashold - 1; hashtab[hashsta].vertnum != ~0; hashsta --) ; /* Start index of first segment to reconsider is last empty slot */
+  hashend = hashold;                              /* First segment to reconsider ends at the end of the old array                            */
+  while (hashend != hashsta) {                    /* For each of the two segments to consider                                                */
+    for (hashnum = hashsta; hashnum < hashend; hashnum ++) { /* Re-compute position of vertices in new table                                 */
+      Gnum                        vertnum;
 
-    for (hashtmp = hashold - 1, hashnew = 0;      /* Temporarily move vertices away from end of old table to prevent overflowing */
-         hashtab[hashtmp].vertnum != ~0; hashtmp --) {
-      while (hashtab[++ hashnew].vertnum != ~0) ; /* Find an empty slot to receive moved vertex */
+      vertnum = hashtab[hashnum].vertnum;
+      if (vertnum != ~0) {                        /* If hash slot used */
+        Gnum                        hashnew;
+
+        for (hashnew = (vertnum * BGRAPHBIPARTFMHASHPRIME) & hashmsk; ; hashnew = (hashnew + 1) & hashmsk) {
+          if (hashnew == hashnum)                 /* If hash slot is the same */
+            break;                                /* There is nothing to do   */
+          if (hashtab[hashnew].vertnum == ~0) {   /* If new slot is empty     */
 #ifdef SCOTCH_DEBUG_BGRAPH2
-      if (hashnew >= hashtmp) {
-        errorPrint ("bgraphBipartFmResize: internal error (2)");
-        return     (1);
-      }
+            if ((hashnew > hashnum) && (hashnew < hashend)) { /* If vertex is not moved either before its old position or after the end of the segment */
+              errorPrint ("bgraphBipartFmResize: internal error (2)");
+              return     (1);
+            }
 #endif /* SCOTCH_DEBUG_BGRAPH2 */
+            hashtab[hashnew] = hashtab[hashnum];  /* Copy data to new slot         */
+            hashtab[hashnum].mswpnum = ~0;        /* TRICK: not tested at creation */
+            hashtab[hashnum].vertnum = ~0;        /* Make old slot empty           */
+            break;
+          }
+        }
 
-      hashtab[hashnew] = hashtab[hashtmp];        /* Move vertex from end of table */
-      hashtab[hashtmp].mswpnum = ~0;              /* TRICK: not tested at creation */
-      hashtab[hashtmp].vertnum = ~0;              /* Set old slot as free          */
-    }
-  }
-
-  for (hashnum = 0; hashnum < hashold; hashnum ++) { /* Re-compute position of vertices in new table */
-    Gnum                        vertnum;
-
-    vertnum = hashtab[hashnum].vertnum;
-    if (vertnum != ~0) {                          /* If hash slot used */
-      Gnum                        hashnew;
-
-      for (hashnew = (vertnum * BGRAPHBIPARTFMHASHPRIME) & hashmsk; ; hashnew = (hashnew + 1) & hashmsk) {
-        if (hashnew == hashnum)                   /* If hash slot is the same      */
-          break;                                  /* There is nothing to do        */
-        if (hashtab[hashnew].vertnum == ~0) {     /* If new slot is empty          */
-          hashtab[hashnew] = hashtab[hashnum];    /* Copy data to new slot         */
-          hashtab[hashnum].mswpnum = ~0;          /* TRICK: not tested at creation */
-          hashtab[hashnum].vertnum = ~0;          /* Make old slot empty           */
-          break;
+        if (hashtab[hashnew].gainlink.next >= BGRAPHBIPARTFMSTATELINK) /* If vertex was linked, re-link it */
+          gainTablAdd (tablptr, &hashtab[hashnew].gainlink, hashtab[hashnew].compgain);
+        else if (hashtab[hashnew].gainlink.next == BGRAPHBIPARTFMSTATEUSED) { /* Re-lock used vertices */
+          hashtab[hashnew].gainlink.prev = (GainLink *) *lockptr; /* Lock it */
+          *lockptr = &hashtab[hashnew];
         }
       }
-      if ((hashnew > hashnum) && (hashnew < hashold)) /* If vertex was an overflowed vertex which will be replaced at end of old table */
-        continue;                                 /* It will be re-processed again and re-linked once for good at the end of the loop  */
-
-      if (hashtab[hashnew].gainlink.next >= BGRAPHBIPARTFMSTATELINK) /* If vertex was linked, re-link it */
-        gainTablAdd (tablptr, &hashtab[hashnew].gainlink, hashtab[hashnew].compgain);
-      else if (hashtab[hashnew].gainlink.next == BGRAPHBIPARTFMSTATEUSED) { /* Re-lock used vertices */
-        hashtab[hashnew].gainlink.prev = (GainLink *) *lockptr; /* Lock it */
-        *lockptr = &hashtab[hashnew];
-      }
     }
-  }
+
+    hashend = hashsta;                            /* End of second segment to consider is start of first one    */
+    hashsta = 0;                                  /* Start of second segment is beginning of array              */
+  }                                               /* After second segment, hashsta = hashend = 0 and loop stops */
 
   for (savenum = 0; savenum < savenbr; savenum ++) {
     Gnum                  vertnum;
@@ -860,7 +860,7 @@ const Gnum                                  commgainextn)
     if (vertnum == ~0)                            /* If unallocated slot */
       continue;                                   /* Skip to next slot   */
 
-    veloval = (grafptr->s.velotax != NULL) ? grafptr->s.velotax[vertnum] : 1;
+    veloval = (velotax != NULL) ? velotax[vertnum] : 1;
     partval = hashtab[hashnum].partval;
     if ((partval < 0) || (partval > 1)) {
       errorPrint ("bgraphBipartFmCheck: invalid vertex part value");
@@ -880,8 +880,8 @@ const Gnum                                  commgainextn)
 
     commcut  =
     commgain = 0;
-    for (edgenum = grafptr->s.verttax[vertnum];   /* For all neighbors */
-         edgenum < grafptr->s.vendtax[vertnum]; edgenum ++) {
+    for (edgenum = verttax[vertnum];              /* For all neighbors */
+         edgenum < vendtax[vertnum]; edgenum ++) {
       Gnum                edloval;
       Gnum                vertend;
       Gnum                hashend;
@@ -889,9 +889,9 @@ const Gnum                                  commgainextn)
       int                 partond;
       int                 partdlt;
 
-      vertend = grafptr->s.edgetax[edgenum];
+      vertend = edgetax[edgenum];
       partond = grafptr->parttax[vertend] ^ swapval;
-      edloval = (grafptr->s.edlotax != NULL) ? grafptr->s.edlotax[edgenum] : 1;
+      edloval = (edlotax != NULL) ? edlotax[edgenum] : 1;
 
       for (hashend = (vertend * BGRAPHBIPARTFMHASHPRIME) & hashmsk; ; hashend = (hashend + 1) & hashmsk) {
         if (hashtab[hashend].vertnum == vertend) { /* If end vertex found */
