@@ -1,4 +1,4 @@
-/* Copyright 2004,2007 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008 ENSEIRB, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -42,6 +42,10 @@
 /**                                 to     07 sep 2001     **/
 /**                # Version 0.1  : from : 14 apr 2001     **/
 /**                                 to     24 mar 2003     **/
+/**                # Version 2.0  : from : 01 jul 2008     **/
+/**                                 to   : 01 jul 2008     **/
+/**                # Version 5.1  : from : 22 nov 2008     **/
+/**                                 to   : 22 nov 2008     **/
 /**                                                        **/
 /************************************************************/
 
@@ -59,6 +63,134 @@
 /*                               */
 /*********************************/
 
+/* This routine keeps track of the amount of
+** allocated memory, and keeps track of the
+** maximum allowed.
+*/
+
+#ifdef COMMON_MEMORY_TRACE
+
+#define COMMON_MEMORY_SKEW          (MAX ((sizeof (size_t)), (sizeof (double)))) /* For proper alignment */
+
+static size_t           memorysiz = 0;            /*+ Number of allocated bytes        +*/
+static size_t           memorymax = 0;            /*+ Maximum amount of allocated data +*/
+
+#if (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD))
+static int              muteflag = 1;             /*+ Flag for mutex initialization +*/
+static pthread_mutex_t  mutelocdat;               /*+ Local mutex for updates       +*/
+#endif /* (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD)) */
+
+void *
+memAllocRecord (
+size_t                      newsiz)
+{
+  size_t              newadd;
+  byte *              newptr;
+
+#if (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD))
+  if (muteflag != 0) {                            /* Unsafe code with respect to race conditions but should work as first allocs are sequential */
+    muteflag = 0;
+    pthread_mutex_init (&mutelocdat, NULL);       /* Initialize local mutex */
+  }
+  pthread_mutex_lock (&mutelocdat);               /* Lock local mutex */
+#endif /* (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD)) */
+
+  if ((newptr = malloc (newsiz + COMMON_MEMORY_SKEW)) == NULL) /* Non-zero sizes will guarantee non-NULL pointers */
+    newadd = (size_t) 0;
+  else {
+    newadd = newsiz;
+    *((size_t *) newptr) = newsiz;                /* Record size for freeing                */
+    newptr += COMMON_MEMORY_SKEW;                 /* Skew pointer while enforcing alignment */
+  }
+
+  memorysiz += newadd;
+  if (memorymax < memorysiz)
+    memorymax = memorysiz;
+
+#if (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD))
+  pthread_mutex_unlock (&mutelocdat);             /* Unlock local mutex */
+#endif /* (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD)) */
+
+  return ((void *) newptr);                       /* Return skewed pointer or NULL */
+}
+
+void *
+memReallocRecord (
+void *                      oldptr,
+size_t                      newsiz)
+{
+  byte *              tmpptr;
+  byte *              newptr;
+  size_t              oldsiz;
+  size_t              newadd;
+
+  tmpptr = ((byte *) oldptr) - COMMON_MEMORY_SKEW;
+  oldsiz = *((size_t *) tmpptr);
+
+#if (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD))
+  pthread_mutex_lock (&mutelocdat);               /* Lock local mutex */
+#endif /* (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD)) */
+
+  if ((newptr = realloc (tmpptr, newsiz + COMMON_MEMORY_SKEW)) == NULL)
+    newadd = (size_t) 0;
+  else {
+    newadd = newsiz - oldsiz;                     /* Add difference between the two sizes   */
+    *((size_t *) newptr) = newsiz;                /* Record size for freeing                */
+    newptr += COMMON_MEMORY_SKEW;                 /* Skew pointer while enforcing alignment */
+  }
+
+  memorysiz += newadd;
+  if (memorymax < memorysiz)
+    memorymax = memorysiz;
+
+#if (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD))
+  pthread_mutex_unlock (&mutelocdat);             /* Unlock local mutex */
+#endif /* (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD)) */
+
+  return ((void *) newptr);                       /* Return skewed pointer or NULL */
+}
+
+void
+memFreeRecord (
+void *                      oldptr)
+{
+  byte *              tmpptr;
+  size_t              oldsiz;
+
+  tmpptr = ((byte *) oldptr) - COMMON_MEMORY_SKEW;
+  oldsiz = *((size_t *) tmpptr);
+
+#if (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD))
+  pthread_mutex_lock (&mutelocdat);               /* Lock local mutex */
+#endif /* (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD)) */
+
+  free (tmpptr);
+  memorysiz -= oldsiz;
+
+#if (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD))
+  pthread_mutex_unlock (&mutelocdat);             /* Unlock local mutex */
+#endif /* (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD)) */
+}
+
+size_t
+memMax ()
+{
+  size_t              curmax;
+
+#if (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD))
+  pthread_mutex_lock (&mutelocdat);               /* Lock local mutex */
+#endif /* (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD)) */
+
+  curmax = memorymax;
+
+#if (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD))
+  pthread_mutex_unlock (&mutelocdat);             /* Unlock local mutex */
+#endif /* (defined (COMMON_PTHREAD) || defined (SCOTCH_PTHREAD)) */
+
+  return (curmax);
+}
+#endif /* COMMON_MEMORY_TRACE */
+
 /* This routine allocates a set of arrays in
 ** a single memAlloc()'ed array, the address
 ** of which is placed in the first argument.
