@@ -212,6 +212,7 @@ const Gnum                  degrlocmax)
     grafptr->proccnttab[procnum]     = grafptr->procdsptab[procnum + 1]     - grafptr->procdsptab[procnum];
   }
 
+  grafptr->flagval |= DGRAPHFREEPRIV;
   return (dgraphBuild3 (grafptr, baseval,
                         vertlocnbr, vertloctax, vendloctax, veloloctax, velolocsum, vnumloctax, vlblloctax,
                         edgelocnbr, edgelocsiz, edgeloctax, edgegsttax, edloloctax, degrlocmax));
@@ -261,19 +262,19 @@ const Gnum                  degrlocmax)
   Gnum                  vertngbmin;               /* Smallest vertex number of neighbor process   */
   Gnum                  vertlocnum;
   Gnum                  edgelocnum;
-  const Gnum *          vlbllocptr;               /* Pointer to current vertex label              */
-  DgraphLablSortVert *  vesongbptr;               /* Pointer to current sort cell                 */
-  DgraphLablSortVert *  vesongbtnd;               /* Pointer to end of current sort array         */
-  DgraphLablSortVert *  vesongbtab[2];            /* Neighbor vertex sorting array                */
-  int                   vesongbnbr[2];            /* Sizes of both vertex sort arrays             */
-  DgraphLablSortEdge *  edsoloctab;               /* Local edge sorting array                     */
-  DgraphLablSortEdge *  edsoloctnd;               /* Pointer to end of edge sort array            */
-  DgraphLablSortEdge *  edsolocptr;               /* Pointer to current sort edge                 */
-  MPI_Request           requloctab[2];            /* Arrays for pipelined communication           */
+  const Gnum *          vlbllocptr;               /* Pointer to current vertex label                  */
+  DgraphLablSortVert *  vesongbptr;               /* Pointer to current sort cell                     */
+  DgraphLablSortVert *  vesongbtnd;               /* Pointer to end of current sort array             */
+  DgraphLablSortVert *  vesongbtab[2];            /* Neighbor vertex sorting array [norestrict:async] */
+  int                   vesongbnbr[2];            /* Sizes of both vertex sort arrays                 */
+  DgraphLablSortEdge *  edsoloctab;               /* Local edge sorting array                         */
+  DgraphLablSortEdge *  edsoloctnd;               /* Pointer to end of edge sort array                */
+  DgraphLablSortEdge *  edsolocptr;               /* Pointer to current sort edge                     */
+  MPI_Request           requloctab[2];            /* Arrays for pipelined communication               */
   MPI_Status            statloctab[2];
-  int                   cheklocval;               /* Local consistency flag                       */
-  int                   chekglbval;               /* Global consistency flag                      */
-  Gnum                  reduloctab[20];           /* Arrays for reductions                        */
+  int                   cheklocval;               /* Local consistency flag                           */
+  int                   chekglbval;               /* Global consistency flag                          */
+  Gnum                  reduloctab[20];           /* Arrays for reductions                            */
   Gnum                  reduglbtab[20];
 
   reduloctab[0]  =   baseval;                     /* Check argument consistency */
@@ -321,7 +322,6 @@ const Gnum                  degrlocmax)
   grafptr->edgeglbsmx = reduglbtab[14];           /* Set maximum size of local edge arrays */
   grafptr->degrglbmax = reduglbtab[15];           /* Set maximum degree                    */
 
-  grafptr->flagval    = DGRAPHNONE;               /* Set graph flags */
   grafptr->baseval    = baseval;
   grafptr->vertglbnbr = reduglbtab[17];           /* Set global and local data */
   grafptr->vertlocnbr = vertlocnbr;
@@ -353,7 +353,7 @@ const Gnum                  degrlocmax)
                        &vesongbtab[0], (size_t) (grafptr->vertglbmax * sizeof (DgraphLablSortVert)),
                        &vesongbtab[1], (size_t) (grafptr->vertglbmax * sizeof (DgraphLablSortVert)),
                        &edsoloctab,    (size_t) (grafptr->edgeglbmax * sizeof (DgraphLablSortEdge)),
-                       NULL) == NULL){
+                       NULL) == NULL) {
       errorPrint ("dgraphBuild3: out of memory");
       return     (1);
     }
