@@ -41,7 +41,7 @@
 /**                a distributed graph.                    **/
 /**                                                        **/
 /**   DATES      : # Version 5.0  : from : 07 feb 2006     **/
-/**                                 to     01 mar 2008     **/
+/**                                 to     17 jun 2008     **/
 /**                                                        **/
 /**   NOTES      : # The definitions of MPI_Gather and     **/
 /**                  MPI_Gatherv indicate that elements in **/
@@ -104,11 +104,12 @@ const int                     protnum)            /* -1 means allgather */
   Gnum                baseval;
   Gnum * restrict     verttax;                    /* Target vertex array for root, dummy for non-roots        */
   Gnum * restrict     velotax;                    /* Target vertex load array for root, dummy for non-roots   */
+  Gnum * restrict     vnumtax;                    /* Target vertex index array for root, dummy for non-roots  */
   Gnum * restrict     vlbltax;                    /* Target vertex label array for root, dummy for non-roots  */
   Gnum * restrict     edgetax;                    /* Target edge array for root, dummy for non-roots          */
   Gnum * restrict     edlotax;                    /* Target edge load array for root, dummy for non-roots     */
   Gnum                vertlocnbr;                 /* Size of temporary distributed vertex array               */
-  Gnum * restrict     vertloctab;                 /* Temporary vertex array if graph is not compact           */
+  Gnum * restrict     vertloctax;                 /* Temporary vertex array if graph is not compact           */
   Gnum                edgelocnbr;                 /* Size of temporary distributed edge array                 */
   Gnum * restrict     edgeloctab;                 /* Temporary edge array if distributed graph is not compact */
   int * restrict      recvcnttab;                 /* Count array for gather operations                        */
@@ -116,6 +117,8 @@ const int                     protnum)            /* -1 means allgather */
   int                 cheklocval;
   int                 chekglbval;
 
+  const Gnum * restrict const edgeloctax = dgrfptr->edgeloctax;
+
 #ifdef SCOTCH_DEBUG_DGRAPH1
   cheklocval = 0;
   if (cgrfptr != NULL)                            /* Centralized graphs should be provided by all */
@@ -143,16 +146,19 @@ const int                     protnum)            /* -1 means allgather */
   cheklocval = 0;
   if (cgrfptr != NULL) {                          /* If root process */
     Gnum                velonbr;
+    Gnum                vnumnbr;
     Gnum                vlblnbr;
     Gnum                edlonbr;
 
     velonbr = (dgrfptr->veloloctax != NULL) ? dgrfptr->vertglbnbr : 0;
+    vnumnbr = (dgrfptr->vnumloctax != NULL) ? dgrfptr->vertglbnbr : 0;
     vlblnbr = (dgrfptr->vlblloctax != NULL) ? dgrfptr->vertglbnbr : 0;
     edlonbr = (dgrfptr->edloloctax != NULL) ? dgrfptr->edgeglbnbr : 0;
 
     if (memAllocGroup ((void **) (void *)
                        &cgrfptr->verttax, (size_t) ((dgrfptr->vertglbnbr + 1) * sizeof (Gnum)),
                        &cgrfptr->velotax, (size_t) (velonbr                   * sizeof (Gnum)),
+                       &cgrfptr->vnumtax, (size_t) (vnumnbr                   * sizeof (Gnum)),
                        &cgrfptr->vlbltax, (size_t) (vlblnbr                   * sizeof (Gnum)), NULL) == NULL) {
       errorPrint ("dgraphGatherAll2: out of memory (1)");
       cheklocval = 1;
@@ -178,7 +184,7 @@ const int                     protnum)            /* -1 means allgather */
     if (memAllocGroup ((void **) (void *)
                        &recvcnttab, (size_t) (dgrfptr->procglbnbr * sizeof (int)), /* Allocated for non-roots too but don't care as these are very small */
                        &recvdsptab, (size_t) (dgrfptr->procglbnbr * sizeof (int)),
-                       &vertloctab, (size_t) (vertlocnbr          * sizeof (Gnum)),
+                       &vertloctax, (size_t) (vertlocnbr          * sizeof (Gnum)),
                        &edgeloctab, (size_t) (edgelocnbr          * sizeof (Gnum)), NULL) == NULL) {
       errorPrint ("dgraphGatherAll2: out of memory (3)");
       cheklocval = 1;
@@ -206,6 +212,7 @@ const int                     protnum)            /* -1 means allgather */
   if (cgrfptr != NULL) {
     verttax = cgrfptr->verttax - baseval;
     velotax = (dgrfptr->veloloctax != NULL) ? (cgrfptr->velotax - baseval) : NULL;
+    vnumtax = (dgrfptr->vnumloctax != NULL) ? (cgrfptr->vnumtax - baseval) : NULL;
     vlbltax = (dgrfptr->vlblloctax != NULL) ? (cgrfptr->vlbltax - baseval) : NULL;
     edgetax = cgrfptr->edgetax - baseval;
     edlotax = (dgrfptr->edloloctax != NULL) ? (cgrfptr->edlotax - baseval) : NULL;
@@ -218,7 +225,7 @@ const int                     protnum)            /* -1 means allgather */
     cgrfptr->vendtax    = verttax + 1;            /* Compact edge array */
     cgrfptr->velotax    = velotax;
     cgrfptr->velosum    = dgrfptr->veloglbsum;
-    cgrfptr->vnumtax    = NULL;                   /* Vertex indices are not relevant */
+    cgrfptr->vnumtax    = vnumtax;
     cgrfptr->vlbltax    = vlbltax;
     cgrfptr->edgenbr    = dgrfptr->edgeglbnbr;
     cgrfptr->edgetax    = edgetax;
@@ -265,11 +272,10 @@ const int                     protnum)            /* -1 means allgather */
     }
   }
   else {                                          /* Distributed graph is not compact */
-    Gnum * restrict     vertloctax;
     Gnum                vertlocnum;
     Gnum * restrict     edgelocptr;
 
-    vertloctax = vertloctab - baseval;
+    vertloctax -= baseval;                        /* Base temporary vertex array   */
     for (vertlocnum = baseval, edgelocptr = edgeloctab; /* Build vertex send array */
          vertlocnum < dgrfptr->vertlocnnd; vertlocnum ++) {
       Gnum                edgelocnum;
@@ -277,10 +283,10 @@ const int                     protnum)            /* -1 means allgather */
       vertloctax[vertlocnum] = dgrfptr->vendloctax[vertlocnum] - dgrfptr->vertloctax[vertlocnum]; /* Get edge counts */
 
       for (edgelocnum = dgrfptr->vertloctax[vertlocnum]; edgelocnum < dgrfptr->vendloctax[vertlocnum]; edgelocnum ++)
-        *edgelocptr ++ = dgrfptr->edgeloctax[edgelocnum];
+        *edgelocptr ++ = edgeloctax[edgelocnum];
     }
 
-    if (dgraphGatherAll3 (vertloctab, dgrfptr->vertlocnbr,
+    if (dgraphGatherAll3 (vertloctax + baseval, dgrfptr->vertlocnbr,
                           verttax + 1,            /* First index will always be equal to baseval, and procdsptab holds based values */
                           dgrfptr->proccnttab, dgrfptr->procdsptab, protnum, dgrfptr->proccomm) != MPI_SUCCESS) {
       errorPrint ("dgraphGatherAll2: communication error (4)");
@@ -310,6 +316,14 @@ const int                     protnum)            /* -1 means allgather */
     if (dgraphGatherAll3 (dgrfptr->veloloctax + baseval, dgrfptr->vertlocnbr,
                           velotax,                /* Based array since procdsptab holds based values */
                           dgrfptr->proccnttab, dgrfptr->procdsptab, protnum, dgrfptr->proccomm) != MPI_SUCCESS) {
+      errorPrint ("dgraphGatherAll2: communication error (6)");
+      return     (1);
+    }
+  }
+  if (dgrfptr->vnumloctax != NULL) {
+    if (dgraphGatherAll3 (dgrfptr->vnumloctax + baseval, dgrfptr->vertlocnbr,
+                          vnumtax,                /* Based array since procdsptab holds based values */
+                          dgrfptr->proccnttab, dgrfptr->procdsptab, protnum, dgrfptr->proccomm) != MPI_SUCCESS) {
       errorPrint ("dgraphGatherAll2: communication error (5)");
       return     (1);
     }
@@ -318,7 +332,7 @@ const int                     protnum)            /* -1 means allgather */
     if (dgraphGatherAll3 (dgrfptr->vlblloctax + baseval, dgrfptr->vertlocnbr,
                           vlbltax,                /* Based array since procdsptab holds based values */
                           dgrfptr->proccnttab, dgrfptr->procdsptab, protnum, dgrfptr->proccomm) != MPI_SUCCESS) {
-      errorPrint ("dgraphGatherAll2: communication error (6)");
+      errorPrint ("dgraphGatherAll2: communication error (7)");
       return     (1);
     }
   }
@@ -346,7 +360,7 @@ const int                     protnum)            /* -1 means allgather */
     if (dgraphGatherAll3 (dgrfptr->edgeloctax + baseval, dgrfptr->edgelocnbr, /* Send global indices */
                           edgetax,                /* Based array as recvdsptab holds based values    */
                           recvcnttab, recvdsptab, protnum, dgrfptr->proccomm) != MPI_SUCCESS) {
-      errorPrint ("dgraphGatherAll2: communication error (7)");
+      errorPrint ("dgraphGatherAll2: communication error (8)");
       return     (1);
     }
 
@@ -354,7 +368,7 @@ const int                     protnum)            /* -1 means allgather */
       if (dgraphGatherAll3 (dgrfptr->edloloctax + baseval, dgrfptr->edgelocnbr,
                             edlotax,              /* Based array as recvdsptab holds based values */
                             recvcnttab, recvdsptab, protnum, dgrfptr->proccomm) != MPI_SUCCESS) {
-        errorPrint ("dgraphGatherAll2: communication error (8)");
+        errorPrint ("dgraphGatherAll2: communication error (9)");
         return     (1);
       }
     }
@@ -363,7 +377,7 @@ const int                     protnum)            /* -1 means allgather */
     if (dgraphGatherAll3 (edgeloctab, dgrfptr->edgelocnbr,
                           edgetax,                /* Based array as recvdsptab holds based values */
                           recvcnttab, recvdsptab, protnum, dgrfptr->proccomm) != MPI_SUCCESS) {
-      errorPrint ("dgraphGatherAll2: communication error (9)");
+      errorPrint ("dgraphGatherAll2: communication error (10)");
       return     (1);
     }
 
@@ -382,7 +396,7 @@ const int                     protnum)            /* -1 means allgather */
       if (dgraphGatherAll3 (edgeloctab, dgrfptr->edgelocnbr, /* Send compacted edge load array    */
                             edlotax,              /* Based array as recvdsptab holds based values */
                             recvcnttab, recvdsptab, protnum, dgrfptr->proccomm) != MPI_SUCCESS) {
-        errorPrint ("dgraphGatherAll2: communication error (10)");
+        errorPrint ("dgraphGatherAll2: communication error (11)");
         return     (1);
       }
     }
@@ -457,7 +471,7 @@ const int                     protnum)            /* -1 means allgather */
 #ifdef SCOTCH_DEBUG_DGRAPH2
   cheklocval = (cgrfptr != NULL) ? graphCheck (cgrfptr) : 0;
   if (MPI_Allreduce (&cheklocval, &chekglbval, 1, MPI_INT, MPI_MAX, dgrfptr->proccomm) != MPI_SUCCESS) {
-    errorPrint ("dgraphGatherAll2: communication error (11)");
+    errorPrint ("dgraphGatherAll2: communication error (12)");
     return     (1);
   }
   if (chekglbval != 0) {
