@@ -1,4 +1,4 @@
-/* Copyright 2007,2008 ENSEIRB, INRIA & CNRS
+/* Copyright 2007-2009 ENSEIRB, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -49,6 +49,8 @@
 /**                                 to     20 jun 1998     **/
 /**                # Version 5.0  : from : 28 feb 2006     **/
 /**                                 to     05 feb 2008     **/
+/**                # Version 5.1  : from : 28 aug 2008     **/
+/**                                 to     19 jan 2009     **/
 /**                                                        **/
 /************************************************************/
 
@@ -70,53 +72,63 @@
 */
 
 #define DGRAPHHALOFILLNAME          dgraphHaloFillGeneric
+#define DGRAPHHALOFILLSIZE          attrglbsiz
 #define DGRAPHHALOFILLCOPY(d,s,n)   memCpy ((d), (s), (n))
 #include "dgraph_halo_fill.c"
 #undef DGRAPHHALOFILLNAME
+#undef DGRAPHHALOFILLSIZE
 #undef DGRAPHHALOFILLCOPY
 
 #define DGRAPHHALOFILLNAME          dgraphHaloFillGnum
+#define DGRAPHHALOFILLSIZE          sizeof (Gnum)
 #define DGRAPHHALOFILLCOPY(d,s,n)   *((Gnum *) (d)) = *((Gnum *) (s))
 #include "dgraph_halo_fill.c"
 #undef DGRAPHHALOFILLNAME
+#undef DGRAPHHALOFILLSIZE
 #undef DGRAPHHALOFILLCOPY
 
 #define DGRAPHHALOFILLNAME          dgraphHaloFillGraphPart
+#define DGRAPHHALOFILLSIZE          sizeof (GraphPart)
 #define DGRAPHHALOFILLCOPY(d,s,n)   *((GraphPart *) (d)) = *((GraphPart *) (s))
 #include "dgraph_halo_fill.c"
 #undef DGRAPHHALOFILLNAME
+#undef DGRAPHHALOFILLSIZE
 #undef DGRAPHHALOFILLCOPY
 
 #define DGRAPHHALOFILLNAME          dgraphHaloFillInt /* In case Gnum is not int */
+#define DGRAPHHALOFILLSIZE          sizeof (int)
 #define DGRAPHHALOFILLCOPY(d,s,n)   *((int *) (d)) = *((int *) (s))
 #include "dgraph_halo_fill.c"
 #undef DGRAPHHALOFILLNAME
+#undef DGRAPHHALOFILLSIZE
 #undef DGRAPHHALOFILLCOPY
 
 static
 void
 dgraphHaloFill (
 const Dgraph * restrict const grafptr,
-const byte * restrict const   attrgsttab,         /* Attribute array to diffuse     */
+const void * restrict const   attrgsttab,         /* Attribute array to diffuse     */
 int                           attrglbsiz,         /* Type extent of attribute       */
 byte * restrict const         attrsndtab,         /* Array for packing data to send */
-int * restrict const          senddsptab,         /* Temporary displacement array   */
+int * const                   senddsptab,         /* Temporary displacement array   */
 const int * restrict const    sendcnttab)         /* Count array                    */
 {
   int                 procnum;
+  byte **             attrdsptab;
 
-  senddsptab[0] = 0;                              /* Pre-set send arrays for send buffer filling routines */
+  attrdsptab = (byte **) senddsptab;              /* TRICK: use senddsptab (int *) as attrdsptab (byte **) */
+  attrdsptab[0] = attrsndtab;                     /* Pre-set send arrays for send buffer filling routines */
   for (procnum = 1; procnum < grafptr->procglbnbr; procnum ++)
-    senddsptab[procnum] = senddsptab[procnum - 1] + sendcnttab[procnum - 1];
+    attrdsptab[procnum] = attrdsptab[procnum - 1] + sendcnttab[procnum - 1] * attrglbsiz;
 
   if (attrglbsiz == sizeof (Gnum))
-    dgraphHaloFillGnum (grafptr, attrgsttab, attrglbsiz, attrsndtab, senddsptab);
+    dgraphHaloFillGnum (grafptr, attrgsttab, attrglbsiz, attrdsptab);
   else if (attrglbsiz == sizeof (GraphPart))
-    dgraphHaloFillGraphPart (grafptr, attrgsttab, attrglbsiz, attrsndtab, senddsptab);
+    dgraphHaloFillGraphPart (grafptr, attrgsttab, attrglbsiz, attrdsptab);
   else if (attrglbsiz == sizeof (int))            /* In case Gnum is not int */
-    dgraphHaloFillInt (grafptr, attrgsttab, attrglbsiz, attrsndtab, senddsptab);
+    dgraphHaloFillInt (grafptr, attrgsttab, attrglbsiz, attrdsptab);
   else                                            /* Generic but slower fallback routine */
-    dgraphHaloFillGeneric (grafptr, attrgsttab, attrglbsiz, attrsndtab, senddsptab);
+    dgraphHaloFillGeneric (grafptr, attrgsttab, attrglbsiz, attrdsptab);
 
   senddsptab[0] = 0;                              /* Pre-set send arrays for data sending routines */
   for (procnum = 1; procnum < grafptr->procglbnbr; procnum ++)
@@ -128,47 +140,37 @@ const int * restrict const    sendcnttab)         /* Count array
 */
 
 #ifdef SCOTCH_DEBUG_DGRAPH2
-static
 int
 dgraphHaloCheck (
 const Dgraph * restrict const grafptr)
 {
-  int * proctmptab;                               /* Array to collect datas */
-  int * dsptab;
-  int * cnttab;
-  int procnum;
+  int *               proctab;                    /* Array to collect data */
+  int                 procnum;
+  int                 o;
 
-  if (memAllocGroup ((void **)
-                     &proctmptab, (size_t) (grafptr->procglbnbr * sizeof (int)),
-                     &dsptab,     (size_t) (grafptr->procglbnbr * sizeof (int)),
-                     &cnttab,     (size_t) (grafptr->procglbnbr * sizeof (int)),NULL) == NULL) {
+  if ((proctab = memAlloc (grafptr->procglbnbr * sizeof (int))) == NULL) {
     errorPrint ("dgraphHaloCheck: out of memory");
-    return (1);
-  }
-
-  memSet (proctmptab, ~0, grafptr->procglbnbr * sizeof (int));
-  for (procnum = 0; procnum < grafptr->procglbnbr; procnum ++) {
-    dsptab[procnum] = procnum;
-    cnttab[procnum] = 1;                          /* One element for each process */
-  }
-
-  if (MPI_Alltoallv (grafptr->procsndtab, cnttab, dsptab, MPI_INT,
-                     proctmptab, cnttab, dsptab, MPI_INT, grafptr->proccomm) != MPI_SUCCESS) {
-    errorPrint ("dgraphHaloCheck: communication error");
-    memFree    (proctmptab);                      /* Free group leader */
     return     (1);
   }
 
+  if (MPI_Alltoall (grafptr->procsndtab, 1, MPI_INT, proctab, 1, MPI_INT, grafptr->proccomm) != MPI_SUCCESS) {
+    errorPrint ("dgraphHaloCheck: communication error");
+    memFree    (proctab);                         /* Free group leader */
+    return     (1);
+  }
+
+  o = 0;
   for (procnum = 0; procnum < grafptr->procglbnbr; procnum ++) {
-    if (proctmptab[procnum] != grafptr->procrcvtab[procnum]) {
+    if (proctab[procnum] != grafptr->procrcvtab[procnum]) {
       errorPrint ("dgraphHaloCheck: data error");
-      memFree    (proctmptab);                    /* Free group leader */
-      return     (1);
+      o = 1;
+      break;
     }
   }
-  memFree (proctmptab);                           /* Free group leader */
-  return  (0);
 
+  memFree (proctab);                              /* Free group leader */
+
+  return  (o);
 }
 #endif /* SCOTCH_DEBUG_DGRAPH2 */
 
@@ -184,7 +186,7 @@ static
 int
 dgraphHaloSync2 (
 Dgraph * restrict const       grafptr,
-byte * restrict const         attrgsttab,         /* Attribute array to share                      */
+void * restrict const         attrgsttab,         /* Attribute array to share                      */
 const MPI_Datatype            attrglbtype,        /* Attribute datatype                            */
 byte ** const                 attrsndptr,         /* Pointer to array for packing data to send     */
 int ** const                  senddspptr,         /* Pointers to communication displacement arrays */
@@ -201,7 +203,7 @@ int ** const                  recvdspptr)
   MPI_Type_extent (attrglbtype, &attrglbsiz);     /* Get type extent */
   if (memAllocGroup ((void **) (void *)
                      attrsndptr, (size_t) (grafptr->procsndnbr * attrglbsiz),
-                     senddspptr, (size_t) (grafptr->procglbnbr * sizeof (int)),
+                     senddspptr, (size_t) (grafptr->procglbnbr * MAX (sizeof (int), sizeof (byte *))), /* TRICK: use senddsptab (int *) as attrdsptab (byte **) */
                      recvdspptr, (size_t) (grafptr->procglbnbr * sizeof (int)), NULL) == NULL) {
     errorPrint ("dgraphHaloSync2: out of memory");
     return     (1);
@@ -226,7 +228,7 @@ int ** const                  recvdspptr)
 int
 dgraphHaloSync (
 Dgraph * restrict const       grafptr,
-byte * restrict const         attrgsttab,         /* Attribute array to share */
+void * restrict const         attrgsttab,         /* Attribute array to share */
 const MPI_Datatype            attrglbtype)        /* Attribute datatype       */
 {
   byte *              attrsndtab;                 /* Array for packing data to send */
@@ -265,14 +267,14 @@ void *
 dgraphHaloAsync2 (
 DgraphHaloRequest * restrict  requptr)
 {
-  return ((void *) dgraphHaloSync (requptr->grafptr, requptr->attrgsttab, requptr->attrglbtype));
+  return ((void *) (intptr_t) dgraphHaloSync (requptr->grafptr, requptr->attrgsttab, requptr->attrglbtype));
 }
 #endif /* SCOTCH_PTHREAD */
 
 void
 dgraphHaloAsync (
 Dgraph * restrict const       grafptr,
-byte * restrict const         attrgsttab,         /* Attribute array to share */
+void * restrict const         attrgsttab,         /* Attribute array to share */
 const MPI_Datatype            attrglbtype,        /* Attribute datatype       */
 DgraphHaloRequest * restrict  requptr)
 {
