@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007-2010 ENSEIRB, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -49,6 +49,8 @@
 /**                                 to   : 06 jan 2006     **/
 /**                # Version 5.0  : from : 26 apr 2006     **/
 /**                                 to   : 03 apr 2008     **/
+/**                # Version 5.1  : from : 16 jun 2008     **/
+/**                                 to   : 15 apr 2010     **/
 /**                                                        **/
 /************************************************************/
 
@@ -85,7 +87,10 @@
 #include "order.h"
 #ifdef SCOTCH_PTSCOTCH
 #include "dgraph.h"
+#include "dgraph_halo.h"
+#include "dmapping.h"
 #include "dorder.h"
+#include "library_dmapping.h"
 #endif /* SCOTCH_PTSCOTCH */
 #include "library_mapping.h"
 #include "library_order.h"
@@ -162,7 +167,7 @@ char *                      argv[])
           exit       (0);
         case 'V' :
           fprintf (stderr, "dummysizes, version %s - F. Pellegrini\n", SCOTCH_VERSION);
-          fprintf (stderr, "Copyright 2004,2007,2008 ENSEIRB, INRIA & CNRS, France\n");
+          fprintf (stderr, "Copyright 2004,2007-2009 ENSEIRB, INRIA & CNRS, France\n");
           fprintf (stderr, "This software is libre/free software under CeCILL-C -- see the user's manual for more information\n");
           return  (0);
         default :
@@ -194,30 +199,36 @@ char *                      argv[])
 #else /* SCOTCH_PTSCOTCH */
   substab[1][1] = "ptscotchf.h";
 #endif /* SCOTCH_PTSCOTCH */
-  substab[2][0] = "DUMMYINT";
-  substab[2][1] = EXPAND(INT);
-  substab[3][0] = "DUMMYMAXINT";
-  substab[3][1] = EXPAND(INTVALMAX);
-  substab[4][0] = "DUMMYPTFLAG";
+  substab[2][0] = "DUMMYIDX";
+  substab[2][1] = EXPAND(IDX);
+  substab[3][0] = "DUMMYINT";
+  substab[3][1] = EXPAND(INT);
+  substab[4][0] = "DUMMYMAXINT";
+  substab[4][1] = EXPAND(INTVALMAX);
+  substab[5][0] = "DUMMYPTFLAG";
 #ifdef SCOTCH_PTSCOTCH
-  substab[4][1] = "PTSCOTCH";
+  substab[5][1] = "PTSCOTCH";
 #else /* SCOTCH_PTSCOTCH */
-  substab[4][1] = "SEQSCOTCH";
+  substab[5][1] = "SEQSCOTCH";
 #endif /* SCOTCH_PTSCOTCH */
-  subsnbr = 5;
-  subsFill (substab[subsnbr ++], "DUMMYSIZEARCH",  sizeof (Arch));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEGEOM",  sizeof (Geom));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEGRAPH", sizeof (Graph));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEMESH",  sizeof (Mesh));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEMAP",   sizeof (LibMapping));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEORDER", sizeof (LibOrder));
-  subsFill (substab[subsnbr ++], "DUMMYSIZESTRAT", sizeof (Strat *));
+  subsnbr = 6;
+  subsFill (substab[subsnbr ++], "DUMMYSIZEARCH",          sizeof (Arch));
+  subsFill (substab[subsnbr ++], "DUMMYSIZEGEOM",          sizeof (Geom));
+  subsFill (substab[subsnbr ++], "DUMMYSIZEGRAPH",         sizeof (Graph));
+  subsFill (substab[subsnbr ++], "DUMMYSIZEMESH",          sizeof (Mesh));
+  subsFill (substab[subsnbr ++], "DUMMYSIZEMAP",           sizeof (LibMapping));
+  subsFill (substab[subsnbr ++], "DUMMYSIZEORDER",         sizeof (LibOrder));
+  subsFill (substab[subsnbr ++], "DUMMYSIZESTRAT",         sizeof (Strat *));
 #ifdef SCOTCH_PTSCOTCH
-  subsFill (substab[subsnbr ++], "DUMMYSIZEDGRAPH", sizeof (Dgraph));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEDORDER", sizeof (Dorder));
+  subsFill (substab[subsnbr ++], "DUMMYSIZEDGRAPHHALOREQ", sizeof (DgraphHaloRequest)); /* TRICK: before DUMMYSIZEDGRAPH */
+  subsFill (substab[subsnbr ++], "DUMMYSIZEDGRAPH",        sizeof (Dgraph));
+  subsFill (substab[subsnbr ++], "DUMMYSIZEDMAP",          sizeof (LibDmapping));
+  subsFill (substab[subsnbr ++], "DUMMYSIZEDORDER",        sizeof (Dorder));
 #else /* SCOTCH_PTSCOTCH */
-  subsFill (substab[subsnbr ++], "DUMMYSIZEDGRAPH", 1);
-  subsFill (substab[subsnbr ++], "DUMMYSIZEDORDER", 1);
+  subsFill (substab[subsnbr ++], "DUMMYSIZEDGRAPHHALOREQ", 1); /* TRICK: before DUMMYSIZEDGRAPH */
+  subsFill (substab[subsnbr ++], "DUMMYSIZEDGRAPH",        1);
+  subsFill (substab[subsnbr ++], "DUMMYSIZEDMAP",          1);
+  subsFill (substab[subsnbr ++], "DUMMYSIZEDORDER",        1);
 #endif /* SCOTCH_PTSCOTCH */
 
   while (fgets (chartab, CHARMAX, C_filepntrhedinp) != NULL) { /* Infinite loop on file lines */
