@@ -1,4 +1,4 @@
-/* Copyright 2007,2008 ENSEIRB, INRIA & CNRS
+/* Copyright 2007-2010 ENSEIRB, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -40,7 +40,9 @@
 /**                libSCOTCH library.                      **/
 /**                                                        **/
 /**   DATES      : # Version 5.0  : from : 25 apr 2006     **/
-/**                                 to     31 may 2008     **/
+/**                                 to     11 nov 2008     **/
+/**                # Version 5.1  : from : 29 mar 2010     **/
+/**                                 to     29 mar 2010     **/
 /**                                                        **/
 /************************************************************/
 
@@ -159,7 +161,7 @@ int
 SCOTCH_dgraphOrderCompute (
 const SCOTCH_Dgraph * const grafptr,              /*+ Graph to order      +*/
 SCOTCH_Dordering * const    ordeptr,              /*+ Ordering to compute +*/
-const SCOTCH_Strat * const  stratptr)             /*+ Ordering strategy   +*/
+SCOTCH_Strat * const        stratptr)             /*+ Ordering strategy   +*/
 {
   return (SCOTCH_dgraphOrderComputeList (grafptr, ordeptr, 0, NULL, stratptr));
 }
@@ -179,7 +181,7 @@ const SCOTCH_Dgraph * const grafptr,              /*+ Graph to order
 SCOTCH_Dordering * const    ordeptr,              /*+ Ordering to compute             +*/
 const SCOTCH_Num            listnbr,              /*+ Number of vertices in list      +*/
 const SCOTCH_Num * const    listtab,              /*+ List of vertex indices to order +*/
-const SCOTCH_Strat * const  stratptr)             /*+ Ordering strategy               +*/
+SCOTCH_Strat * const        stratptr)             /*+ Ordering strategy               +*/
 {
   Dorder *            srcordeptr;                 /* Pointer to ordering          */
   DorderCblk *        srccblkptr;                 /* Initial column block         */
@@ -190,7 +192,7 @@ const SCOTCH_Strat * const  stratptr)             /*+ Ordering strategy
   const Strat *       ordstratptr;                /* Pointer to ordering strategy */
 
   if (*((Strat **) stratptr) == NULL)             /* Set default ordering strategy if necessary */
-    *((Strat **) stratptr) = stratInit (&hdgraphorderststratab, "n{sep=m{asc=b{width=3,strat=q{strat=f}},low=q{strat=h},vert=1000,dvert=100,dlevl=0,proc=1,seq=q{strat=m{type=h,vert=100,low=h{pass=10},asc=b{width=3,bnd=f{bal=0.2},org=h{pass=10}f{bal=0.2}}}}},ole=s,ose=s,osq=n{sep=/(vert>120)?m{type=h,vert=100,low=h{pass=10},asc=b{width=3,bnd=f{bal=0.2},org=h{pass=10}f{bal=0.2}}};,ole=f{cmin=15,cmax=100000,frat=0.0},ose=g}}");
+    *((Strat **) stratptr) = stratInit (&hdgraphorderststratab, "n{sep=m{asc=b{width=3,strat=q{strat=f}},low=q{strat=h},vert=100,dvert=10,dlevl=0,proc=1,seq=q{strat=m{type=h,vert=100,low=h{pass=10},asc=b{width=3,bnd=f{bal=0.2},org=h{pass=10}f{bal=0.2}}}}},ole=q{strat=n{sep=/(vert>120)?m{type=h,vert=100,low=h{pass=10},asc=b{width=3,bnd=f{bal=0.2},org=h{pass=10}f{bal=0.2}}};,ole=f{cmin=15,cmax=100000,frat=0.0},ose=g}},ose=s,osq=n{sep=/(vert>120)?m{type=h,vert=100,low=h{pass=10},asc=b{width=3,bnd=f{bal=0.2},org=h{pass=10}f{bal=0.2}}};,ole=f{cmin=15,cmax=100000,frat=0.0},ose=g}}");
   ordstratptr = *((Strat **) stratptr);
   if (ordstratptr->tabl != &hdgraphorderststratab) {
     errorPrint ("SCOTCH_dgraphOrderComputeList: not a distributed ordering strategy");
