@@ -1,4 +1,4 @@
-/* Copyright 2004,2007 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008 ENSEIRB, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -46,7 +46,7 @@
 /**                # Version 4.0  : from : 11 dec 2001     **/
 /**                                 to     09 dec 2005     **/
 /**                # Version 5.0  : from : 10 sep 2006     **/
-/**                                 to     07 jun 2007     **/
+/**                                 to     03 apr 2008     **/
 /**                                                        **/
 /************************************************************/
 
@@ -214,7 +214,7 @@ const SCOTCH_Num * const    edlotab)              /* Edge load array
   srcgrafptr->vertnbr = vertnbr;
   srcgrafptr->vertnnd = vertnbr + baseval;
   srcgrafptr->verttax = (Gnum *) verttab - baseval;
-  srcgrafptr->vendtax = ((vendtab == NULL) || (vendtab == verttab + 1)) ? srcgrafptr->verttax + 1 : (Gnum *) vendtab - baseval;
+  srcgrafptr->vendtax = ((vendtab == NULL) || (vendtab == verttab) || (vendtab == verttab + 1)) ? srcgrafptr->verttax + 1 : (Gnum *) vendtab - baseval;
   srcgrafptr->velotax = ((velotab == NULL) || (velotab == verttab)) ? NULL : (Gnum *) velotab - baseval;
   srcgrafptr->vnumtax = NULL;
   srcgrafptr->vlbltax = ((vlbltab == NULL) || (vlbltab == verttab)) ? NULL : (Gnum *) vlbltab - baseval;
@@ -471,7 +471,7 @@ double *                    edlodltptr)
   edlodlt = 0.0L;
   if (srcgrafptr->edgenbr > 0) {
     if (srcgrafptr->edlotax != NULL) {            /* If graph has edge loads */
-      edlomin = INT_MAX;
+      edlomin = GNUMMAX;
       edlomax = 0;
       edlosum = 0;
 
