@@ -1,4 +1,4 @@
-/* Copyright 2004,2007 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007-2009 ENSEIRB, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -47,12 +47,27 @@
 /**                # Version 4.0  : from : 12 dec 2001     **/
 /**                                 to     24 nov 2005     **/
 /**                # Version 5.0  : from : 24 feb 2007     **/
-/**                                 to     23 feb 2008     **/
+/**                                 to     24 jul 2007     **/
+/**                # Version 5.1  : from : 25 oct 2007     **/
+/**                                 to     27 apr 2009     **/
 /**                                                        **/
 /************************************************************/
 
 #define MODULE_H
 
+/*
+** Collective communication handling.
+*/
+
+#ifdef SCOTCH_PTOP
+#ifndef SCOTCH_COLLECTIVE_TEST
+#define SCOTCH_COLLECTIVE_TEST      0             /* Never perform collective communication for data exchange */
+#endif /* SCOTCH_COLLECTIVE_TEST */
+#endif /* SCOTCH_PTOP */
+#ifndef SCOTCH_COLLECTIVE_TEST
+#define SCOTCH_COLLECTIVE_TEST      1             /* Always perform collective communication for data exchange */
+#endif /* SCOTCH_COLLECTIVE_TEST */
+
 /*
 ** Debug values.
 */
@@ -61,19 +76,23 @@
 #ifndef SCOTCH_DEBUG
 #define SCOTCH_DEBUG
 #endif /* SCOTCH_DEBUG */
+
 #define SCOTCH_DEBUG_ARCH2
+#define SCOTCH_DEBUG_GAIN2
+#define SCOTCH_DEBUG_PARSER2
+#define SCOTCH_DEBUG_BDGRAPH2
 #define SCOTCH_DEBUG_BGRAPH2
 #define SCOTCH_DEBUG_DGRAPH2
+#define SCOTCH_DEBUG_DMAP2
 #define SCOTCH_DEBUG_DORDER2
-#define SCOTCH_DEBUG_GAIN2
 #define SCOTCH_DEBUG_GEOM2
 #define SCOTCH_DEBUG_GRAPH2
 #define SCOTCH_DEBUG_HDGRAPH2
 #define SCOTCH_DEBUG_HGRAPH2
 #define SCOTCH_DEBUG_HMESH2
+#define SCOTCH_DEBUG_KDGRAPH2
 #define SCOTCH_DEBUG_KGRAPH2
 #define SCOTCH_DEBUG_LIBRARY2
-#define SCOTCH_DEBUG_MAIN2
 #define SCOTCH_DEBUG_MAP2
 #define SCOTCH_DEBUG_MESH2
 #define SCOTCH_DEBUG_ORDER2
@@ -85,18 +104,21 @@
 
 #ifdef SCOTCH_DEBUG
 #define SCOTCH_DEBUG_ARCH1
+#define SCOTCH_DEBUG_GAIN1
+#define SCOTCH_DEBUG_PARSER1
+#define SCOTCH_DEBUG_BDGRAPH1
 #define SCOTCH_DEBUG_BGRAPH1
 #define SCOTCH_DEBUG_DGRAPH1
+#define SCOTCH_DEBUG_DMAP1
 #define SCOTCH_DEBUG_DORDER1
-#define SCOTCH_DEBUG_GAIN1
 #define SCOTCH_DEBUG_GEOM1
 #define SCOTCH_DEBUG_GRAPH1
 #define SCOTCH_DEBUG_HDGRAPH1
 #define SCOTCH_DEBUG_HGRAPH1
 #define SCOTCH_DEBUG_HMESH1
+#define SCOTCH_DEBUG_KDGRAPH1
 #define SCOTCH_DEBUG_KGRAPH1
 #define SCOTCH_DEBUG_LIBRARY1
-#define SCOTCH_DEBUG_MAIN1
 #define SCOTCH_DEBUG_MAP1
 #define SCOTCH_DEBUG_MESH1
 #define SCOTCH_DEBUG_ORDER1
@@ -113,7 +135,7 @@
 #if ((! defined SCOTCH_COMMON_EXTERNAL) || (defined SCOTCH_COMMON_RENAME))
 #define clockGet                    _SCOTCHclockGet
 
-#define fileNameDistExpand          _SCOTCHfileNameDistExpand 
+#define commonStubDummy             _SCOTCHcommonStubDummy
 
 #define usagePrint                  _SCOTCHusagePrint
 
@@ -121,6 +143,15 @@
 #define errorPrintW                 SCOTCH_errorPrintW
 #define errorProg                   SCOTCH_errorProg
 
+#define fileBlockClose              _SCOTCHfileBlockClose
+#define fileBlockOpen               _SCOTCHfileBlockOpen
+#define fileBlockOpenDist           _SCOTCHfileBlockOpenDist
+#define fileCompress                _SCOTCHfileCompress
+#define fileCompressType            _SCOTCHfileCompressType
+#define fileUncompress              _SCOTCHfileUncompress
+#define fileUncompressType          _SCOTCHfileUncompressType
+#define fileNameDistExpand          _SCOTCHfileNameDistExpand 
+
 #define intLoad                     _SCOTCHintLoad
 #define intSave                     _SCOTCHintSave
 #define intAscn                     _SCOTCHintAscn
@@ -128,7 +159,6 @@
 #define intRandReset                _SCOTCHintRandReset
 #define intRandInit                 _SCOTCHintRandInit
 /* #define intRandVal               _SCOTCHintRandVal Already a macro */
-#define intSearchDicho              _SCOTCHintSearchDicho
 #define intSort1asc1                _SCOTCHintSort1asc1
 #define intSort2asc1                _SCOTCHintSort2asc1
 #define intSort2asc2                _SCOTCHintSort2asc2
@@ -158,6 +188,7 @@
 #define archDomDist                 _SCOTCHarchDomDist
 #define archDomFrst                 _SCOTCHarchDomFrst
 #define archDomBipart               _SCOTCHarchDomBipart
+#define archDomMpiType              _SCOTCHarchDomMpiType
 #endif /* SCOTCH_DEBUG_ARCH2 */
 #define archBuild                   _SCOTCHarchBuild
 #define archCmpltArchLoad           _SCOTCHarchCmpltArchLoad
@@ -165,12 +196,13 @@
 #define archCmpltDomNum             _SCOTCHarchCmpltDomNum
 #define archCmpltDomTerm            _SCOTCHarchCmpltDomTerm
 #define archCmpltDomSize            _SCOTCHarchCmpltDomSize
-#define archCmpltDomWght            _SCOTCHarchCmpltDomWght
+/* #define archCmpltDomWght            _SCOTCHarchCmpltDomWght Already a macro */
 #define archCmpltDomDist            _SCOTCHarchCmpltDomDist
 #define archCmpltDomFrst            _SCOTCHarchCmpltDomFrst
 #define archCmpltDomLoad            _SCOTCHarchCmpltDomLoad
 #define archCmpltDomSave            _SCOTCHarchCmpltDomSave
 #define archCmpltDomBipart          _SCOTCHarchCmpltDomBipart
+#define archCmpltDomMpiType         _SCOTCHarchCmpltDomMpiType
 #define archCmpltwArchBuild         _SCOTCHarchCmpltwArchBuild
 #define archCmpltwArchFree          _SCOTCHarchCmpltwArchFree
 #define archCmpltwArchLoad          _SCOTCHarchCmpltwArchLoad
@@ -184,6 +216,7 @@
 #define archCmpltwDomLoad           _SCOTCHarchCmpltwDomLoad
 #define archCmpltwDomSave           _SCOTCHarchCmpltwDomSave
 #define archCmpltwDomBipart         _SCOTCHarchCmpltwDomBipart
+#define archCmpltwDomMpiType        _SCOTCHarchCmpltwDomMpiType
 #define archDecoArchBuild           _SCOTCHarchDecoArchBuild
 #define archDecoArchFree            _SCOTCHarchDecoArchFree
 #define archDecoArchLoad            _SCOTCHarchDecoArchLoad
@@ -197,34 +230,37 @@
 #define archDecoDomLoad             _SCOTCHarchDecoDomLoad
 #define archDecoDomSave             _SCOTCHarchDecoDomSave
 #define archDecoDomBipart           _SCOTCHarchDecoDomBipart
+#define archDecoDomMpiType          _SCOTCHarchDecoDomMpiType
 #define archHcubArchLoad            _SCOTCHarchHcubArchLoad
 #define archHcubArchSave            _SCOTCHarchHcubArchSave
 #define archHcubDomNum              _SCOTCHarchHcubDomNum
 #define archHcubDomTerm             _SCOTCHarchHcubDomTerm
 #define archHcubDomSize             _SCOTCHarchHcubDomSize
-#define archHcubDomWght             _SCOTCHarchHcubDomWght
+/* #define archHcubDomWght             _SCOTCHarchHcubDomWght Already a macro */
 #define archHcubDomDist             _SCOTCHarchHcubDomDist
 #define archHcubDomFrst             _SCOTCHarchHcubDomFrst
 #define archHcubDomLoad             _SCOTCHarchHcubDomLoad
 #define archHcubDomSave             _SCOTCHarchHcubDomSave
 #define archHcubDomBipart           _SCOTCHarchHcubDomBipart
+#define archHcubDomMpiType          _SCOTCHarchHcubDomMpiType
 #define archTleafArchLoad           _SCOTCHarchTleafArchLoad
 #define archTleafArchSave           _SCOTCHarchTleafArchSave
 #define archTleafDomNum             _SCOTCHarchTleafDomNum
 #define archTleafDomTerm            _SCOTCHarchTleafDomTerm
 #define archTleafDomSize            _SCOTCHarchTleafDomSize
-#define archTleafDomWght            _SCOTCHarchTleafDomWght
+/* #define archTleafDomWght            _SCOTCHarchTleafDomWght Already a macro */
 #define archTleafDomDist            _SCOTCHarchTleafDomDist
 #define archTleafDomFrst            _SCOTCHarchTleafDomFrst
 #define archTleafDomLoad            _SCOTCHarchTleafDomLoad
 #define archTleafDomSave            _SCOTCHarchTleafDomSave
 #define archTleafDomBipart          _SCOTCHarchTleafDomBipart
+#define archTleafDomMpiType         _SCOTCHarchTleafDomMpiType
 #define archMesh2ArchLoad           _SCOTCHarchMesh2ArchLoad
 #define archMesh2ArchSave           _SCOTCHarchMesh2ArchSave
 #define archMesh2DomNum             _SCOTCHarchMesh2DomNum
 #define archMesh2DomTerm            _SCOTCHarchMesh2DomTerm
 #define archMesh2DomSize            _SCOTCHarchMesh2DomSize
-#define archMesh2DomWght            _SCOTCHarchMesh2DomWght
+/* #define archMesh2DomWght            _SCOTCHarchMesh2DomWght Already a macro */
 #define archMesh2DomDist            _SCOTCHarchMesh2DomDist
 #define archMesh2DomFrst            _SCOTCHarchMesh2DomFrst
 #define archMesh2DomLoad            _SCOTCHarchMesh2DomLoad
@@ -232,74 +268,99 @@
 #define archMesh2DomBipart          _SCOTCHarchMesh2DomBipart
 #define archMesh2DomBipartO         _SCOTCHarchMesh2DomBipartO
 #define archMesh2DomBipartU         _SCOTCHarchMesh2DomBipartU
+#define archMesh2DomMpiType         _SCOTCHarchMesh2DomMpiType
 #define archMesh3ArchLoad           _SCOTCHarchMesh3ArchLoad
 #define archMesh3ArchSave           _SCOTCHarchMesh3ArchSave
 #define archMesh3DomNum             _SCOTCHarchMesh3DomNum
 #define archMesh3DomTerm            _SCOTCHarchMesh3DomTerm
 #define archMesh3DomSize            _SCOTCHarchMesh3DomSize
-#define archMesh3DomWght            _SCOTCHarchMesh3DomWght
+/* #define archMesh3DomWght            _SCOTCHarchMesh3DomWght Already a macro */
 #define archMesh3DomDist            _SCOTCHarchMesh3DomDist
 #define archMesh3DomFrst            _SCOTCHarchMesh3DomFrst
 #define archMesh3DomLoad            _SCOTCHarchMesh3DomLoad
 #define archMesh3DomSave            _SCOTCHarchMesh3DomSave
 #define archMesh3DomBipart          _SCOTCHarchMesh3DomBipart
+#define archMesh3DomMpiType         _SCOTCHarchMesh3DomMpiType
 #define archTermArchLoad            _SCOTCHarchTermArchLoad
 #define archTermArchSave            _SCOTCHarchTermArchSave
 #define archTermDomNum              _SCOTCHarchTermDomNum
 #define archTermDomTerm             _SCOTCHarchTermDomTerm
 #define archTermDomSize             _SCOTCHarchTermDomSize
-#define archTermDomWght             _SCOTCHarchTermDomWght
+/* #define archTermDomWght             _SCOTCHarchTermDomWght Already a macro */
 #define archTermDomDist             _SCOTCHarchTermDomDist
 #define archTermDomFrst             _SCOTCHarchTermDomFrst
 #define archTermDomLoad             _SCOTCHarchTermDomLoad
 #define archTermDomSave             _SCOTCHarchTermDomSave
 #define archTermDomBipart           _SCOTCHarchTermDomBipart
+#define archTermDomMpiType          _SCOTCHarchTermDomMpiType
 #define archTorus2ArchLoad          _SCOTCHarchTorus2ArchLoad
 #define archTorus2ArchSave          _SCOTCHarchTorus2ArchSave
 #define archTorus2DomNum            _SCOTCHarchTorus2DomNum
 #define archTorus2DomTerm           _SCOTCHarchTorus2DomTerm
 #define archTorus2DomSize           _SCOTCHarchTorus2DomSize
-#define archTorus2DomWght           _SCOTCHarchTorus2DomWght
+/* #define archTorus2DomWght           _SCOTCHarchTorus2DomWght Already a macro */
 #define archTorus2DomDist           _SCOTCHarchTorus2DomDist
 #define archTorus2DomFrst           _SCOTCHarchTorus2DomFrst
 #define archTorus2DomBipart         _SCOTCHarchTorus2DomBipart
 #define archTorus2DomLoad           _SCOTCHarchTorus2DomLoad
 #define archTorus2DomSave           _SCOTCHarchTorus2DomSave
 #define archTorus2DomBipart         _SCOTCHarchTorus2DomBipart
+#define archTorus2DomMpiType        _SCOTCHarchTorus2DomMpiType
 #define archTorus3ArchLoad          _SCOTCHarchTorus3ArchLoad
 #define archTorus3ArchSave          _SCOTCHarchTorus3ArchSave
 #define archTorus3DomNum            _SCOTCHarchTorus3DomNum
 #define archTorus3DomTerm           _SCOTCHarchTorus3DomTerm
 #define archTorus3DomSize           _SCOTCHarchTorus3DomSize
-#define archTorus3DomWght           _SCOTCHarchTorus3DomWght
+/* #define archTorus3DomWght           _SCOTCHarchTorus3DomWght Already a macro */
 #define archTorus3DomDist           _SCOTCHarchTorus3DomDist
 #define archTorus3DomFrst           _SCOTCHarchTorus3DomFrst
 #define archTorus3DomLoad           _SCOTCHarchTorus3DomLoad
 #define archTorus3DomSave           _SCOTCHarchTorus3DomSave
 #define archTorus3DomBipart         _SCOTCHarchTorus3DomBipart
-#define archVcmpltArchLoad          _SCOTCHarchVcmpltArchLoad
-#define archVcmpltArchSave          _SCOTCHarchVcmpltArchSave
+#define archTorus3DomMpiType        _SCOTCHarchTorus3DomMpiType
+/* #define archVcmpltArchLoad          _SCOTCHarchVcmpltArchLoad Already a macro */
+/* #define archVcmpltArchSave          _SCOTCHarchVcmpltArchSave Already a macro */
 #define archVcmpltDomNum            _SCOTCHarchVcmpltDomNum
 #define archVcmpltDomTerm           _SCOTCHarchVcmpltDomTerm
 #define archVcmpltDomSize           _SCOTCHarchVcmpltDomSize
-#define archVcmpltDomWght           _SCOTCHarchVcmpltDomWght
+/* #define archVcmpltDomWght           _SCOTCHarchVcmpltDomWght Already a macro */
 #define archVcmpltDomDist           _SCOTCHarchVcmpltDomDist
 #define archVcmpltDomFrst           _SCOTCHarchVcmpltDomFrst
 #define archVcmpltDomBipart         _SCOTCHarchVcmpltDomBipart
 #define archVcmpltDomLoad           _SCOTCHarchVcmpltDomLoad
 #define archVcmpltDomSave           _SCOTCHarchVcmpltDomSave
 #define archVcmpltDomBipart         _SCOTCHarchVcmpltDomBipart
-#define archVhcubArchLoad           _SCOTCHarchVhcubArchLoad
-#define archVhcubArchSave           _SCOTCHarchVhcubArchSave
+#define archVcmpltDomMpiType        _SCOTCHarchVcmpltDomMpiType
+/* #define archVhcubArchLoad           _SCOTCHarchVhcubArchLoad Already a macro */
+/* #define archVhcubArchSave           _SCOTCHarchVhcubArchSave Already a macro */
 #define archVhcubDomNum             _SCOTCHarchVhcubDomNum
 #define archVhcubDomTerm            _SCOTCHarchVhcubDomTerm
 #define archVhcubDomSize            _SCOTCHarchVhcubDomSize
-#define archVhcubDomWght            _SCOTCHarchVhcubDomWght
+/* #define archVhcubDomWght            _SCOTCHarchVhcubDomWght Already a macro */
 #define archVhcubDomDist            _SCOTCHarchVhcubDomDist
 #define archVhcubDomFrst            _SCOTCHarchVhcubDomFrst
 #define archVhcubDomLoad            _SCOTCHarchVhcubDomLoad
 #define archVhcubDomSave            _SCOTCHarchVhcubDomSave
 #define archVhcubDomBipart          _SCOTCHarchVhcubDomBipart
+#define archVhcubDomMpiType         _SCOTCHarchVhcubDomMpiType
+
+#define bdgraphInit                 _SCOTCHbdgraphInit
+#define bdgraphInit2                _SCOTCHbdgraphInit2
+#define bdgraphExit                 _SCOTCHbdgraphExit
+#define bdgraphZero                 _SCOTCHbdgraphZero
+#define bdgraphbipartststratab      _SCOTCHbdgraphbipartststratab
+#define bdgraphCheck                _SCOTCHbdgraphCheck
+#define bdgraphGatherAll            _SCOTCHbdgraphGatherAll
+#define bdgraphBipartBd             _SCOTCHbdgraphBipartBd
+#define bdgraphBipartDf             _SCOTCHbdgraphBipartDf
+#define bdgraphBipartMl             _SCOTCHbdgraphBipartMl
+#define bdgraphBipartSq             _SCOTCHbdgraphBipartSq
+#define bdgraphBipartSt             _SCOTCHbdgraphBipartSt
+#define bdgraphBipartZr             _SCOTCHbdgraphBipartZr
+#define bdgraphStoreInit            _SCOTCHbdgraphStoreInit
+#define bdgraphStoreExit            _SCOTCHbdgraphStoreExit
+#define bdgraphStoreSave            _SCOTCHbdgraphStoreSave
+#define bdgraphStoreUpdt            _SCOTCHbdgraphStoreUpdt
 
 #define bgraphbipartststratab       _SCOTCHbgraphbipartststratab
 #define bgraphInit                  _SCOTCHbgraphInit
@@ -311,6 +372,7 @@
 #define bgraphZero                  _SCOTCHbgraphZero
 #define bgraphBipartBd              _SCOTCHbgraphBipartBd
 #define bgraphBipartDf              _SCOTCHbgraphBipartDf
+#define bgraphBipartDf2             _SCOTCHbgraphBipartDf2
 #define bgraphBipartEx              _SCOTCHbgraphBipartEx
 #define bgraphBipartFm              _SCOTCHbgraphBipartFm
 #define bgraphBipartGg              _SCOTCHbgraphBipartGg
@@ -331,6 +393,7 @@
 #define dgraphBuildGrid3D           _SCOTCHdgraphBuildGrid3D
 #define dgraphBuildHcub             _SCOTCHdgraphBuildHcub
 #define dgraphCheck                 _SCOTCHdgraphCheck
+#define dgraphBand                  _SCOTCHdgraphBand
 #define dgraphCoarsen               _SCOTCHdgraphCoarsen
 #define dgraphExit                  _SCOTCHdgraphExit
 #define dgraphFold                  _SCOTCHdgraphFold
@@ -341,18 +404,37 @@
 #define dgraphGather                _SCOTCHdgraphGather
 #define dgraphGatherAll             _SCOTCHdgraphGatherAll
 #define dgraphGatherAll2            _SCOTCHdgraphGatherAll2
-#define dgraphGhst                  _SCOTCHdgraphGhst
+/* #define dgraphGhst                  _SCOTCHdgraphGhst Already a macro        */
+/* #define dgraphGhstReplace           _SCOTCHdgraphGhstReplace Already a macro */
+#define dgraphGhst2                 _SCOTCHdgraphGhst2
 #define dgraphHaloSync              _SCOTCHdgraphHaloSync
+#define dgraphHaloAsync             _SCOTCHdgraphHaloAsync
+#define dgraphHaloWait              _SCOTCHdgraphHaloWait
+#define dgraphHaloCheck             _SCOTCHdgraphHaloCheck
 #define dgraphInduceList            _SCOTCHdgraphInduceList
+#define dgraphInducePart            _SCOTCHdgraphInducePart
 #define dgraphInit                  _SCOTCHdgraphInit
 #define dgraphLoad                  _SCOTCHdgraphLoad
+#define dgraphMatchInit             _SCOTCHdgraphMatchInit
+#define dgraphMatchExit             _SCOTCHdgraphMatchExit
+#define dgraphMatchSync             _SCOTCHdgraphMatchSync
+#define dgraphMatchSyncColl         _SCOTCHdgraphMatchSyncColl
+#define dgraphMatchSyncPtop         _SCOTCHdgraphMatchSyncPtop
 #define dgraphMatchCheck            _SCOTCHdgraphMatchCheck
-#define dgraphMatchConvertExit      _SCOTCHdgraphMatchConvertExit
-#define dgraphMatchConvertInit      _SCOTCHdgraphMatchConvertInit
+#define dgraphMatchSc               _SCOTCHdgraphMatchSc
+#define dgraphMatchHy               _SCOTCHdgraphMatchHy
+#define dgraphMatchLc               _SCOTCHdgraphMatchLc
+#define dgraphMatchLy               _SCOTCHdgraphMatchLy
 #define dgraphSave                  _SCOTCHdgraphSave
 #define dgraphScatter               _SCOTCHdgraphScatter
 #define dgraphView                  _SCOTCHdgraphView
 
+#define dmapInit                    _SCOTCHdmapInit
+#define dmapExit                    _SCOTCHdmapExit
+#define dmapAdd                     _SCOTCHdmapAdd
+#define dmapTerm                    _SCOTCHdmapTerm
+#define dmapSave                    _SCOTCHdmapSave
+
 #define dorderDispose               _SCOTCHdorderDispose
 #define dorderExit                  _SCOTCHdorderExit
 #define dorderFree                  _SCOTCHdorderFree
@@ -365,6 +447,12 @@
 #define dorderNewSequIndex          _SCOTCHdorderNewSequIndex
 #define dorderPerm                  _SCOTCHdorderPerm
 #define dorderSave                  _SCOTCHdorderSave
+#define dorderSaveBlock             _SCOTCHdorderSaveBlock
+#define dorderSaveMap               _SCOTCHdorderSaveMap
+#define dorderSaveTree              _SCOTCHdorderSaveTree
+#define dorderSaveTree2             _SCOTCHdorderSaveTree2
+#define dorderCblkDist              _SCOTCHdorderCblkDist
+#define dorderTreeDist              _SCOTCHdorderTreeDist
 
 #define gainTablAddLin              _SCOTCHgainTablAddLin
 #define gainTablAddLog              _SCOTCHgainTablAddLog
@@ -394,9 +482,11 @@
 #define graphInducePart             _SCOTCHgraphInducePart
 #define graphGeomLoadChac           _SCOTCHgraphGeomLoadChac
 #define graphGeomLoadHabo           _SCOTCHgraphGeomLoadHabo
+#define graphGeomLoadMmkt           _SCOTCHgraphGeomLoadMmkt
 #define graphGeomLoadScot           _SCOTCHgraphGeomLoadScot
 #define graphGeomSaveChac           _SCOTCHgraphGeomSaveChac
 #define graphGeomSaveScot           _SCOTCHgraphGeomSaveScot
+#define graphGeomSaveMmkt           _SCOTCHgraphGeomSaveMmkt
 #define graphPtscotch               _SCOTCHgraphPtscotch
 
 #define hallOrderHdHalmd            _SCOTCHhallOrderHdHalmd
@@ -414,6 +504,8 @@
 #define hdgraphInduceList           _SCOTCHhdgraphInduceList
 #define hdgraphOrderNd              _SCOTCHhdgraphOrderNd
 #define hdgraphOrderSi              _SCOTCHhdgraphOrderSi
+#define hdgraphOrderSq              _SCOTCHhdgraphOrderSq
+#define hdgraphOrderSq2             _SCOTCHhdgraphOrderSq2
 #define hdgraphOrderSt              _SCOTCHhdgraphOrderSt
 
 #define hgraphorderststratab        _SCOTCHhgraphorderststratab
@@ -450,10 +542,24 @@
 #define hmeshOrderSi                _SCOTCHhmeshOrderSi
 #define hmeshOrderSt                _SCOTCHhmeshOrderSt
 
+#define kdgraphmapststratab         _SCOTCHkdgraphmapststratab
+#define kdgraphInit                 _SCOTCHkdgraphInit
+#define kdgraphExit                 _SCOTCHkdgraphExit
+#define kdgraphMapRb                _SCOTCHkdgraphMapRb
+#define kdgraphMapRbAdd2            _SCOTCHkdgraphMapRbAdd2
+#define kdgraphMapRbAddBoth         _SCOTCHkdgraphMapRbAddBoth
+#define kdgraphMapRbAddOne          _SCOTCHkdgraphMapRbAddOne
+#define kdgraphMapRbAddPart         _SCOTCHkdgraphMapRbAddPart
+#define kdgraphMapRbMap             _SCOTCHkdgraphMapRbMap
+#define kdgraphMapRbPart            _SCOTCHkdgraphMapRbPart
+#define kdgraphMapSt                _SCOTCHkdgraphMapSt
+
 #define kgraphmapststratab          _SCOTCHkgraphmapststratab
 #define kgraphInit                  _SCOTCHkgraphInit
 #define kgraphExit                  _SCOTCHkgraphExit
 #define kgraphMapRb                 _SCOTCHkgraphMapRb
+#define kgraphMapRbMap              _SCOTCHkgraphMapRbMap
+#define kgraphMapRbPart             _SCOTCHkgraphMapRbPart
 #define kgraphMapSt                 _SCOTCHkgraphMapSt
 
 #define listInit                    _SCOTCHlistInit
@@ -466,6 +572,7 @@
 #define listCopy                    _SCOTCHlistCopy
 
 #define mapInit                     _SCOTCHmapInit
+#define mapInit2                    _SCOTCHmapInit2
 #define mapExit                     _SCOTCHmapExit
 #define mapLoad                     _SCOTCHmapLoad
 #define mapSave                     _SCOTCHmapSave
@@ -527,6 +634,7 @@
 #define vdgraphGatherAll            _SCOTCHvdgraphGatherAll
 #define vdgraphInit                 _SCOTCHvdgraphInit
 #define vdgraphSeparateBd           _SCOTCHvdgraphSeparateBd
+#define vdgraphSeparateDf           _SCOTCHvdgraphSeparateDf
 #define vdgraphSeparateMl           _SCOTCHvdgraphSeparateMl
 #define vdgraphSeparateSq           _SCOTCHvdgraphSeparateSq
 #define vdgraphSeparateSt           _SCOTCHvdgraphSeparateSt
