@@ -40,6 +40,8 @@
 /**                                                        **/
 /**   DATES      : # Version 5.0  : from : 23 jun 2007     **/
 /**                                 to   : 16 jun 2008     **/
+/**                # Version 5.1  : from : 26 oct 2008     **/
+/**                                 to   : 26 oct 2008     **/
 /**                                                        **/
 /************************************************************/
 
@@ -113,19 +115,13 @@ char *              argv[])
 
 #ifdef SCOTCH_PTHREAD
   thrdlvlreqval = MPI_THREAD_MULTIPLE;
-  if (MPI_Init_thread (&argc, &argv, thrdlvlreqval, &thrdlvlproval) != MPI_SUCCESS) {
+  if (MPI_Init_thread (&argc, &argv, thrdlvlreqval, &thrdlvlproval) != MPI_SUCCESS)
     errorPrint ("main: Cannot initialize (1)");
-    exit       (1);
-  }
-  if (thrdlvlreqval > thrdlvlproval) {
+  if (thrdlvlreqval > thrdlvlproval)
     errorPrint ("main: MPI implementation is not thread-safe: recompile without SCOTCH_PTHREAD");
-    exit       (1);
-  }
 #else /* SCOTCH_PTHREAD */
-  if (MPI_Init (&argc, &argv) != MPI_SUCCESS) {
+  if (MPI_Init (&argc, &argv) != MPI_SUCCESS)
     errorPrint ("main: Cannot initialize (2)");
-    exit       (1);
-  }
 #endif /* SCOTCH_PTHREAD */
 
   MPI_Comm_size (MPI_COMM_WORLD, &procglbnbr);    /* Get communicator data */
@@ -147,10 +143,8 @@ char *              argv[])
         ((argv[i][0] != '-') || (argv[i][1] == '\0'))) {
       if (C_fileNum < C_FILEARGNBR)               /* A file name has been given */
         C_fileTab[C_fileNum ++].name = argv[i];
-      else {
+      else
         errorPrint ("main: too many file names given");
-        return     (1);
-      }
     }
     else {                                        /* If found an option name */
       switch (argv[i][1]) {
@@ -169,10 +163,8 @@ char *              argv[])
           protglbnum = atoi (&argv[i][2]);
           if ((protglbnum < 0)           ||
               (protglbnum >= procglbnbr) ||
-              ((protglbnum == 0) && (argv[i][2] != '0'))) {
+              ((protglbnum == 0) && (argv[i][2] != '0')))
             errorPrint ("main: invalid root process number");
-            return     (1);
-          }
           break;
         case 'V' :
         case 'v' :
@@ -182,7 +174,6 @@ char *              argv[])
           return  (0);
         default :
           errorPrint ("main: unprocessed option (\"%s\")", argv[i]);
-          return     (1);
       }
     }
   }
