@@ -1,4 +1,4 @@
-/* Copyright 2004,2007 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008 ENSEIRB, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -42,7 +42,7 @@
 /**   DATES      : # Version 4.0  : from : 27 jan 2004     **/
 /**                                 to   : 28 nov 2005     **/
 /**                # Version 5.0  : from : 25 jun 2007     **/
-/**                                 to   : 25 jul 2007     **/
+/**                                 to   : 16 mar 2008     **/
 /**                                                        **/
 /**   NOTES      : # The cost analysis routine leaves the  **/
 /**                  memory management to malloc and free  **/
@@ -58,6 +58,7 @@
 
 #define GOTST
 
+#include "module.h"
 #include "common.h"
 #include "scotch.h"
 #include "gotst.h"
@@ -110,10 +111,9 @@ char *                      argv[])
 
   for (i = 0; i < C_FILENBR; i ++)                /* Set default stream pointers */
     C_fileTab[i].pntr = (C_fileTab[i].mode[0] == 'r') ? stdin : stdout;
-  for (i = 1; i < argc; i ++) {                   /* Loop for all option codes */
-    if ((argv[i][0] != '+') &&                    /* If found a file name      */
-        ((argv[i][0] != '-') || (argv[i][1] == '\0'))) {
-      if (C_fileNum < C_FILEARGNBR)               /* A file name has been given */
+  for (i = 1; i < argc; i ++) {                   /* Loop for all option codes                        */
+    if ((argv[i][0] != '-') || (argv[i][1] == '\0') || (argv[i][1] == '.')) { /* If found a file name */
+      if (C_fileNum < C_FILEARGNBR)               /* File name has been given                         */
         C_fileTab[C_fileNum ++].name = argv[i];
       else {
         errorPrint ("main: too many file names given");
@@ -128,7 +128,7 @@ char *                      argv[])
           return     (0);
         case 'V' :
           fprintf (stderr, "gotst, version %s - F. Pellegrini\n", SCOTCH_VERSION);
-          fprintf (stderr, "Copyright 2004,2007 ENSEIRB, INRIA & CNRS, France\n");
+          fprintf (stderr, "Copyright 2004,2007,2008 ENSEIRB, INRIA & CNRS, France\n");
           fprintf (stderr, "This software is libre/free software under CeCILL-C -- see the user's manual for more information\n");
           return  (0);
         default :
@@ -138,15 +138,7 @@ char *                      argv[])
     }
   }
 
-  for (i = 0; i < C_FILENBR; i ++) {             /* For all file names     */
-    if ((C_fileTab[i].name[0] != '-') ||         /* If not standard stream */
-        (C_fileTab[i].name[1] != '\0')) {
-      if ((C_fileTab[i].pntr = fopen (C_fileTab[i].name, C_fileTab[i].mode)) == NULL) { /* Open the file */
-        errorPrint ("main: cannot open file (%d)", i);
-        return     (1);
-      }
-    }
-  }
+  fileBlockOpen (C_fileTab, C_FILENBR);           /* Open all files */
 
   SCOTCH_graphInit (&grafdat);
   SCOTCH_graphLoad (&grafdat, C_filepntrgrfinp, -1, 3);
@@ -157,7 +149,7 @@ char *                      argv[])
     return     (1);
   }
 #endif /* SCOTCH_DEBUG_ALL */
-  if (memAllocGroup ((void **)
+  if (memAllocGroup ((void **) (void *)
                      &peritab, (size_t) (vertnbr * sizeof (SCOTCH_Num)),
                      &permtab, (size_t) (vertnbr * sizeof (SCOTCH_Num)), NULL) == NULL) {
     errorPrint ("main: out of memory");
@@ -172,19 +164,15 @@ char *                      argv[])
 
   factorView (baseval, vertnbr, verttab, edgenbr, edgetab, permtab, peritab, C_filepntrdatout);
 
-#ifdef SCOTCH_DEBUG_ALL
+  fileBlockClose (C_fileTab, C_FILENBR);          /* Always close explicitely to end eventual (un)compression tasks */
+
   memFree               (peritab);
   SCOTCH_graphOrderExit (&grafdat, &ordedat);
   SCOTCH_graphExit      (&grafdat);
 
-  for (i = 0; i < C_FILENBR; i ++) {             /* For all file names     */
-    if ((C_fileTab[i].name[0] != '-') ||         /* If not standard stream */
-        (C_fileTab[i].name[1] != '\0')) {
-      fclose (C_fileTab[i].pntr);                /* Close the stream */
-    }
-  }
-#endif /* SCOTCH_DEBUG_ALL */
-
+#ifdef COMMON_PTHREAD
+  pthread_exit ((void *) 0);                      /* Allow potential (un)compression tasks to complete */
+#endif /* COMMON_PTHREAD */
   return (0);
 }
 
@@ -217,7 +205,7 @@ FILE * restrict const         stream)
   SCOTCH_Num              vertnum;
   int                     o;
 
-  if (memAllocGroup ((void **)
+  if (memAllocGroup ((void **) (void *)
                      &ldadtab, (size_t) (vertnbr * sizeof (SCOTCH_Num)),
                      &lsontab, (size_t) (vertnbr * sizeof (SCOTCH_Num)),
                      &lbrotab, (size_t) (vertnbr * sizeof (SCOTCH_Num)),
@@ -293,7 +281,7 @@ SCOTCH_Num * restrict         fnnztax)
 
   memSet (lsontax + baseval, ~0, vertnbr * sizeof (SCOTCH_Num)); /* Assume columns have no sons at all */
 
-  if (memAllocGroup ((void **)
+  if (memAllocGroup ((void **) (void *)
                      &frowtab, (size_t) ((vertnbr + 1) * sizeof (SCOTCH_Num)),
                      &fnxttab, (size_t) ((vertnbr + 1) * sizeof (SCOTCH_Num)),
                      &facttax, (size_t) (vertnbr       * sizeof (SCOTCH_Num *)), NULL) == NULL) {
@@ -364,7 +352,7 @@ SCOTCH_Num * restrict         fnnztax)
         foldidx          = frownbr ++;
       }
 
-      memFree (srowtab);                          /* Free now useless factored column */
+      memFree ((void *) srowtab);                 /* Free now useless factored column */
 #ifdef SCOTCH_DEBUG_ALL
       facttax[scolnum] = NULL;
 #endif /* SCOTCH_DEBUG_ALL */
