@@ -42,6 +42,8 @@
 /**                                 to   : 06 jan 2006     **/
 /**                # Version 5.0  : from : 22 jan 2008     **/
 /**                                 to   : 16 mar 2008     **/
+/**                # Version 5.1  : from : 08 sep 2008     **/
+/**                                 to   : 08 sep 2008     **/
 /**                                                        **/
 /************************************************************/
 
@@ -76,7 +78,6 @@ static const char *         C_usageList[] = {
   "  -t<file>   : Save partitioning tree data to <file>",
   "  -V         : Print program version and copyright",
   "  -v<verb>   : Set verbose mode to <verb> :",
-  "                 o  : ordering information",
   "                 s  : strategy information",
   "                 t  : timing information",
   NULL };
